function [StructOut]=CleanHFSpatialStats(StructIn,CleaningParameters)
%
%  [StructOut]=CleanHFSpatialStats(StructIn,CleaningParameters);
%
%  This function cleans the HF Data (either radials or totals).  Sets
%  values that do not meet the criteria set by the user to NaN.
%
%  Only one set of radials from one site can be passed (see GetHFTemporalStats for more
%  explanation).  Or a totals structure.
%  
%  Structin is the original HF radar velocity structure from teh toolbox,
%  with fields added under the field SpatialStats - see GetHFSpatialStats 
%  for more definition.
%
%  CleaningParameters is a cell array of parameters to use when cleaning.
%  The fields to clean by must correspond to the fields under SpatialStats.
%  An example of a CleaningParameters input is:  (Note:  different from the
%  first order statistics cleaning parameters structure!!!!!!)
%  CleaningParameters={'Yes',[20 100],'SumDist3','RangeGood';'No',[2000],'SumAngle2p5','AboveBad'};
%  
%  Format of Cleaning Parameters is:
%     {NaNSwitch,[parameters],'StatisticName','UseOfStatistic'; etc.}
%   size of cell array CleaningParameters will be nx4, where n is the
%   number of flags set
%
%  NOTE:  The "bad" stats themselves are not set to NaN, just the velocity
%  measurements.
%
%  NaNSwitch = 'Yes' means to set to NaNs those velocities where the
%  statistic is undefined (equal to NaN).  'No' means to not do this.
%
%           UseOfStatistic - how to flag.  Valid Entries are:
%               BelowBad - radials with statistic below (not below and equal to)
%                   parameter are bad
%               AboveBad - radials with statistic above parameter are bad
%               RangeGood - radials with statistic in range are good
%               RangeBad - radials statistic in range are bad
%
%
%  If a given field is not found in the HF Velocity structure, a warning is
%  printed to the screen.  Processing does not stop, the field is simply
%  skipped.
%
%  CMH Nov 2007
%
[NumFlagsSet,NumCols]=size(CleaningParameters);
%
StructOut=StructIn;
%
for jflag=1:NumFlagsSet,
    if isfield(StructOut.SpatialStats,CleaningParameters{jflag,3}),
        eval(['TemSer=StructOut.SpatialStats.' CleaningParameters{jflag,3} ';'])
        clear FlagInd
        switch upper(CleaningParameters{jflag,4})
            case 'BELOWBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find(TemSer<FlagVal);
            case 'ABOVEBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find(TemSer>FlagVal);
            case 'RANGEGOOD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find((TemSer<FlagVal(1))|(TemSer>FlagVal(2)));
            case 'RANGEBAD'
                FlagVal=CleaningParameters{jflag,2};
                FlagInd=find((TemSer>FlagVal(1))&(TemSer<FlagVal(2)));
        end
        if numel(FlagInd)>0,
            StructOut.U(FlagInd)=NaN;
            StructOut.V(FlagInd)=NaN;
            if isfield(StructOut,'RadComp'),   % to account for the fact that we can pass radial structures
                StructOut.RadComp(FlagInd)=NaN;
            end
        end
        clear FlagInd
        %
        if isequal(upper(CleaningParameters{jflag,1}),'YES'),
            FlagInd=find(isnan(TemSer));
            if numel(FlagInd)>0,
                StructOut.U(FlagInd)=NaN;
                StructOut.V(FlagInd)=NaN;
                if isfield(StructOut,'RadComp'),   % to account for the fact that we can pass radial structures
                    StructOut.RadComp(FlagInd)=NaN;
                end
            end
        end
        %
    else
        TemStr=['The field ' CleaningParameters{jflag,3} ' Not Found'];
        disp(TemStr)
        TemStr='Cleaning Continuing';
        disp(TemStr)
        TemStr='   ';
        disp(TemStr)
    end
end
%
%  the routine is done
%
return;
%
