function [RcatOut]=CombineSRAndLRRadialsVer2(RcatLongRange,RcatStandardRange,LRMask,SRMask,NewRes,MaxRangeDiff,LogFileDir,LogFileName)
%
%  [RcatOut]=CombineSRAndLRRadialsVer2(RcatLongRange,RcatStandardRange,LRMask,S
%  RMask,NewRes,MaxRangeDiff,LogFileDir,LogFileName);
%
%  This version uses average to get the new radials (not interp).  It will
%  be much slower!
%
%  Uses interp, not average to get grids prior to combining with averaging!  So most often used to get "higher res" LR,
%  and combine with SR
%
%  new grid calculated internally, uses masks to do this
%
%  can pass NaNs for the radial structures if no valid radials.  Can use to
%  interpolate single structures on a common grid then.
%
%  LogFileDir and LogFileName don't have to be specified if we don;t need a comments log
%  file.  Either specify both or none!
%
%
%  Take a little more care with the grid definition  (so we don't have too
%  many).  Make it a true radial grid now.
%
if isstruct(RcatLongRange),
    TemSite=RcatLongRange.SiteOrigin;
end
%
if ~exist('TemSite','var'),
    if isstruct(RcatStandardRange),
        TemSite=RcatStandardRange.SiteOrigin;
    else
        %
        %  no data in the sites
        %
        RcatOut=NaN;
        return;
    end
end
%
[s,a12,a21]=m_idist([LRMask(:,1);SRMask(:,1)],[LRMask(:,2);SRMask(:,2)],TemSite(1),TemSite(2),'wgs84');
s=s/1000;
%
TemRange=NewRes:NewRes:(nanmax(s)+(2*NewRes));
TemBear=0:5:360;  % using 5 degree angle separation
%
[TemRange,TemBear] = meshgrid(TemRange,TemBear);  % bear in true convention, not math, here
%
% Generate equivalent lon, lat and headings using m_fdist
[TemLon,TemLat,head] = m_fdist(TemSite(1),TemSite(2),TemBear,1000*TemRange,'wgs84');
TemLon(TemLon>180)=TemLon(TemLon>180)-360;  % western hemisphere
TemLon=reshape(TemLon,numel(TemLon),1);
TemLat=reshape(TemLat,numel(TemLat),1);
%
IndicesCheckTem1=inpolygon(TemLon,TemLat,LRMask(:,1),LRMask(:,2));
IndicesCheckTem2=inpolygon(TemLon,TemLat,SRMask(:,1),SRMask(:,2));
%
IndicesCheckTem1=IndicesCheckTem1+IndicesCheckTem2;
clear IndicesCheckTem2
%
temmark=find(IndicesCheckTem1>0);
if numel(temmark)>0,
    TemLon=TemLon(temmark);
    TemLat=TemLat(temmark);
else
    disp('grids defined incorrectly')
    disp('routine ending')
    RcatOut=NaN;
    return;
end
%
%  mask the radials in case that has not happened
%
if isstruct(RcatLongRange),
    [RcatLongRange,I] = maskRadials(RcatLongRange,LRMask);
end
%
if isstruct(RcatStandardRange),
    [RcatStandardRange,I] = maskRadials(RcatStandardRange,SRMask);
end
%
if exist('LogFileDir','var'),
    AddLineToLog(LogFileDir,LogFileName,['obtaining regridded data for first set of radials']);
end
%
disp('obtaining regridded data for first set of radials')
pause(1);
%
if exist('LogFileDir','var'),
    RcatLongRange=GetNewRadialsAverage(RcatLongRange,TemLon,TemLat,MaxRangeDiff,LogFileDir,LogFileName);
else
    RcatLongRange=GetNewRadialsAverage(RcatLongRange,TemLon,TemLat,MaxRangeDiff);  % returning NaN (instead of a structure) is o.k.  
end
%
if exist('LogFileDir','var'),
    AddLineToLog(LogFileDir,LogFileName,['obtaining regridded data for second set of radials']);
end
%
disp('obtaining regridded data for second set of radials')
pause(1);
%
if exist('LogFileDir','var'),
    RcatStandardRange=GetNewRadialsAverage(RcatStandardRange,TemLon,TemLat,MaxRangeDiff,LogFileDir,LogFileName);  % NaN just means no valid radials in structure
else
    RcatStandardRange=GetNewRadialsAverage(RcatStandardRange,TemLon,TemLat,MaxRangeDiff);  % NaN just means no valid radials in structure
end
%
if exist('LogFileDir','var'),
    AddLineToLog(LogFileDir,LogFileName,['Combining both sets of radials']);
end
%
RcatOut=SetUpOutputRadials(RcatLongRange,RcatStandardRange);  % both are on the same grid at this point
%
return;
%
function [RcatTem]=GetNewRadialsAverage(RcatIn,LonIn,LatIn,MaxRangeDiff,LogFileDir,LogFileName)
%
%  to get radials on a new grid
%  Note that some of the old measurements will be NaNs  - take care when
%  interpolating
%
if ~isstruct(RcatIn),
    RcatTem=NaN;
    return;
end
%
if isfield(RcatIn,'TempStats'),
    RcatIn=rmfield(RcatIn,'TempStats');
end
%
if isfield(RcatIn,'SpatialStats'),
    RcatIn=rmfield(RcatIn,'SpatialStats');
end
%
ThetaConv=pi/180;
%
RcatTem=RcatIn;
RcatTem.LonLat=[LonIn LatIn];
%
[NumOldGrids,NumOldTimes]=size(RcatIn.U);
RcatTem.U=NaN*ones(numel(LonIn),NumOldTimes);
RcatTem.V=NaN*ones(numel(LonIn),NumOldTimes);
RcatTem.RadComp=NaN*ones(numel(LonIn),NumOldTimes);
RcatTem.Error=NaN*ones(numel(LonIn),NumOldTimes);
RcatTem.Flag=NaN*ones(numel(LonIn),NumOldTimes);
RcatTem.RangeBearHead=NaN*ones(numel(LonIn),3);
%
[range,bear,head]=m_idist(RcatTem.SiteOrigin(1),RcatTem.SiteOrigin(2),RcatTem.LonLat(:,1),...
    RcatTem.LonLat(:,2),'wgs84');
%
head = true2math( head );
bear = true2math( bear );
%
range=range/1000;
%
RcatTem.RangeBearHead=[reshape(range,numel(range),1) reshape(bear,numel(bear),1) reshape(head,numel(head),1)];
%
%  NaNs can mess up the interpolation in certain cases!  So must loop and
%  be careful.
%
%  set up interpolation matrices
%
%  this is where things will get tricky and slow
%
%  Do point by point to save memory.  Damn slow, though.
%
for jpoint=1:numel(LonIn),
    %
    if isequal(rem(jpoint,100),0),
        %
        disp(['Averaging radial data, point ' num2str(jpoint) ' of ' num2str(numel(LonIn))])
    end
    %
    [s,a12,a21]=m_idist(LonIn(jpoint),LatIn(jpoint),RcatIn.LonLat(:,1),RcatIn.LonLat(:,2),'wgs84');
    %
    %
    %
    %  NaNs will occur if points are too close
    %
    temmark=find(isnan(s));
    if numel(temmark)>0,
        s(temmark)=0;
        a12(temmark)=0;
        a21(temmark)=0;
    end
    %
    s=s/1000;  % now in km
    %
    clear temmark
    %
    temmark=find(s<=MaxRangeDiff);
    %
    if ~isempty(temmark),
        %
        MeanUVels=nanmean(RcatIn.U(temmark,:),1);
        MeanVVels=nanmean(RcatIn.V(temmark,:),1);
        %
        LonLatNow=RcatTem.LonLat(jpoint,:);
        LonLatNow=repmat(LonLatNow,numel(MeanUVels),1);
        %
        [ProjectedVectors]=GetProjectedRadialComponents(LonLatNow,MeanUVels',MeanVVels',RcatTem.SiteOrigin);
        %
        RcatTem.RadComp(jpoint,:)=ProjectedVectors';
        %
        %  the errors are standard deviations, so care must be taken when averaging those
        %
        TemSerErr=RcatIn.Error(temmark,:);
        TemSerErr=(TemSerErr.*TemSerErr);
        TemSerErr=sqrt(nansum(TemSerErr,1));
        %
        TemNumPts=sum(~isnan(RcatIn.Error(temmark,:)),1);
        %
        TemSerErrMark=find(isnan(TemSerErr));
        %
        TemSerErr=TemSerErr./TemNumPts;
        %
        if numel(TemSerErrMark)>0,
            TemSerErr(TemSerErrMark)=NaN;
        end
        %
        RcatTem.Error(jpoint,:)= TemSerErr;
        %
        clear TemSerErr TemSerErrMark
        %
    end
end
%
TemHead=repmat(RcatTem.RangeBearHead(:,3),1,numel(RcatTem.TimeStamp));
%
RcatTem.U=RcatTem.RadComp.*cos(ThetaConv*TemHead);
RcatTem.V=RcatTem.RadComp.*sin(ThetaConv*TemHead);
RcatTem.Flag=isfinite(RcatTem.RadComp);
%
%
return;
%
function [RcatOut]=SetUpOutputRadials(Rcat1,Rcat2)
%
%  [RcatOut]=SetUpOutputRadials(Rcat1,Rcat2);
%
%  both are on the same grid at this point, but may not have the same
%  timestamps or number of measurements
%
%  both may or may not be structures (have valid data)
%
%  SiteOrigin should be close enough in both cases to not matter, and
%  range/bear/heading should be close enough also
%
if (~isstruct(Rcat1))&&(~isstruct(Rcat2)),
    RcatOut=NaN;
    return;
end
%
if ~isstruct(Rcat1),
    RcatOut=Rcat2;
    return;
end
%
if ~isstruct(Rcat2),
    RcatOut=Rcat1;
    return;
end
%
%  we now have both sets of radials to combine
%
%  assume that the "best" resolution is hourly
%
%  They do not have too have the same valid hours
%
RcatOut=Rcat1;
%
DateVec1=datevec(Rcat1.TimeStamp);
temmark=find((DateVec1(:,5)>58)&(DateVec1(:,6)>58));
if numel(temmark)>0,
    DateVec1(temmark,4)=DateVec1(temmark,4)+1;
    DateVec1(temmark,5)=0;
    DateVec1(temmark,6)=0;
end
DateVec1=round(DateVec1);
%
DateVec2=datevec(Rcat2.TimeStamp);
temmark=find((DateVec2(:,5)>58)&(DateVec2(:,6)>58));
if numel(temmark)>0,
    DateVec2(temmark,4)=DateVec2(temmark,4)+1;
    DateVec2(temmark,5)=0;
    DateVec2(temmark,6)=0;
end
DateVec2=round(DateVec2);
%
DateVecAll=unique([DateVec1;DateVec2],'rows');  % result is sorted
%
RcatOut.TimeStamp=datenum(DateVecAll);
RcatOut.TimeStamp=RcatOut.TimeStamp';
NumTimes=numel(RcatOut.TimeStamp);
%
[NumGrids,NumCols]=size(RcatOut.LonLat);
%
RcatOut.RadComp=NaN*ones(NumGrids,NumTimes,2);
RcatOut.U=NaN*ones(NumGrids,NumTimes,2);
RcatOut.V=NaN*ones(NumGrids,NumTimes,2);
RcatOut.Error=NaN*ones(NumGrids,NumTimes,2);
%
[C,IA,IB]=intersect(DateVec1,DateVecAll,'rows');
%
%  we can average components as well b/c we only have mesurements in the
%  radial direction
%
IA=IA'; % transpose just to make the point
IB=IB';
RcatOut.RadComp(:,IB,1)=Rcat1.RadComp(:,IA);
RcatOut.Error(:,IB,1)=Rcat1.Error(:,IA);
RcatOut.U(:,IB,1)=Rcat1.U(:,IA);
RcatOut.V(:,IB,1)=Rcat1.V(:,IA);
%
[C,IA,IB]=intersect(DateVec2,DateVecAll,'rows');
%
%  we can average components as well b/c we only have mesurements in the
%  radial direction
%
IA=IA'; % transpose just to make the point
IB=IB';
RcatOut.RadComp(:,IB,2)=Rcat2.RadComp(:,IA);
RcatOut.Error(:,IB,2)=Rcat2.Error(:,IA);
RcatOut.U(:,IB,2)=Rcat2.U(:,IA);
RcatOut.V(:,IB,2)=Rcat2.V(:,IA);
%
RcatOut.RadComp=nanmean(RcatOut.RadComp,3);
%RcatOut.Error=nanmean(RcatOut.Error,3);  (modified, see following code)
RcatOut.U=nanmean(RcatOut.U,3);
RcatOut.V=nanmean(RcatOut.V,3);
%
%  the errors are standard deviations, so we must take care with those
%
NumNonNan=sum(~isnan(RcatOut.Error),3);
TemmarkNaN=find(~NumNonNan);  % check this 
RcatOut.Error=(RcatOut.Error).*(RcatOut.Error);
RcatOut.Error=nansum(RcatOut.Error,3);
RcatOut.Error=sqrt(RcatOut.Error);
RcatOut.Error=RcatOut.Error./NumNonNan;
if numel(TemmarkNaN)>0,
    RcatOut.Error(TemmarkNaN)=NaN;
end
%
RcatOut.Flag=isfinite(RcatOut.RadComp);
%
RcatOut.FileName=[];
RcatOut.FileName=cell(1,numel(RcatOut.TimeStamp));
%
return;
%
