function [CoverageOut]=FindHFBoundingRegion(StructIn,TimeStep)
%
%  [CoverageOut]=FindHFBoundingRegion(StructIn,TimeStep);
%
%  To find the polygon that bounds the HF Radar measurements at a given
%  timestep.  A polygon that encloses the measurements that are non-NAN is returned.
%  If no measurements that are NonNaN are found, then a NaN is returned.  Format of
%  CoverageOut is 2 columns, first column is lon, second column is lat.
%  StructIn can be a TUV structure or a radial structure (or any
%  structure with the fields LonLat, U, and V).
%
%  The first point is repeated at the start and end so that the function
%  inpolygon can be used directly after this 
%
%  !!!  This function will not account for cases where the measurements
%  extend across the zero meridian.  Also will not account for cases where
%  measurements extend across +/- 180 deg long., and are in both east and west
%  longitude.  Best if either east or west longitude is used for all
%  specified positions.
%
%  !!!  latitude assumed in deg north, although degree south could probably be used
%  if the measurements do not cross the equator
%
%  !!!  does not account for holes in the coverage
%
%  CMH, 26 Nov 2007
%
%  Step 1 - find the measurements (velocities and corresponding positions) that are nonNaN
%
TemmarkVel=find((~isnan(StructIn.U(:,TimeStep)))&(~isnan(StructIn.V(:,TimeStep))));
%
if numel(TemmarkVel)<1,
    CoverageOut=NaN;
    return;
end
%
PosOut=StructIn.LonLat(TemmarkVel,:);
%
%  Step 2 - We will eventually sort by latitude, and there is some "slop" in the HF
%  grid specifications.  Let us assume that latitudes within 0.25 km of one
%  another are the same.  Find those places and set to the same latitude.
%  Use one lon for this calculation.
%
PosOutNew=NaN*ones(size(PosOut));
LatGridSlop=0.25;
%
for jpos=1:numel(PosOut(:,1)),
    if isnan(PosOutNew(jpos,1)),  %  means that the new position has not been set yet
        CurrentLon=PosOut(jpos,1);
        CurrentLat=PosOut(jpos,2);
        %
        %  following odd snippet added to make latitudes all just slightly
        %  different b/c m_idist has some sort of odd bug in it.  Can
        %  return NaNs when positions are too close or the same.
        %
        AddToLat=0.000000001;
        [s,a12,a21] = m_idist(CurrentLon,CurrentLat,CurrentLon*ones(size(PosOut(:,1))),PosOut(:,2)+AddToLat,'wgs84');
        s=s/1000;  % now in km
        TemmarkPos=find(abs(s)<=LatGridSlop);  % abs is redundant at the moment
        TemmarkNotSet=find(isnan(PosOutNew(:,1)));
        TemmarkSetNow=intersect(TemmarkPos,TemmarkNotSet);
        if numel(TemmarkSetNow)>0,
            PosOutNew(TemmarkSetNow,1)=PosOut(TemmarkSetNow,1);
            PosOutNew(TemmarkSetNow,2)=CurrentLat;
        end
    end
end
%
%  Step 3a - obtain the latitudes, make sure that we have more than one
%  latitude to search over.  If not, move them up and down slightly by
%  gridslop and duplicate so that we get a polygon returned.
%
[AllLat]=unique(PosOutNew(:,2));
if isequal(numel(AllLat),1),
    MeanLon=mean(PosOutNew(:,1));
    [s,a12,a21] = m_idist(MeanLon,AllLat-0.5,MeanLon,AllLat+0.5,'wgs84'); % deg per km of lat
    s=abs(s)/1000;  % we now have how many km each degree of lat correponds to
    DegPerKm=1/s;
    MovePoints=LatGridSlop*DegPerKm;
    MovePoints=0.5*MovePoints;  % total distance apart if 0.25 km, so each point is moved half that
    NewLon=[PosOutNew(:,1);PosOutNew(:,1)];
    NewLat=[PosOutNew(:,2)+MovePoints;PosOutNew(:,2)-MovePoints];
    PosOutNew=[NewLon NewLat];
    [AllLat]=unique(PosOutNew(:,2));
end
%
%  Step 3b - then find the westernmost and
%  eastermost bounds at each individual latitude
%
EastBounds=NaN*ones(size(AllLat));
WestBounds=NaN*ones(size(AllLat));
%
for jlat=1:numel(AllLat),
    Temmark=find(PosOutNew(:,2)==AllLat(jlat));
    TemLon=PosOutNew(Temmark,1);
    EastBounds(jlat)=max(TemLon);
    WestBounds(jlat)=min(TemLon);
    if isequal(EastBounds(jlat),WestBounds(jlat)),
        %
        %  move the points across in longitude by gridslop.  Use
        %  LatGridSlop, for the move in longitude, since it has already
        %  been defined
        %
        [s,a12,a21] = m_idist(EastBounds(jlat),AllLat(jlat),EastBounds(jlat)+1,AllLat(jlat),'wgs84');
        s=abs(s)/1000;  % we now have how many km each degree of lon correponds to at this latitude
        DegPerKm=1/s;
        MovePoints=LatGridSlop*DegPerKm;
        MovePoints=0.5*MovePoints;  % total distance apart if 0.25 km, so each point is moved half that
        EastBounds(jlat)=EastBounds(jlat)+MovePoints;
        WestBounds(jlat)=WestBounds(jlat)-MovePoints;
        %
    end
end
%
%  Step 4 - Set up the polygon.  Polygon must be ordered - that is, the
%  points must be specified either CW or CCW for use in other functions
%  like inpolygon.
%
LatOut=[AllLat;flipud(AllLat);AllLat(1)];
LonOut=[WestBounds;flipud(EastBounds);WestBounds(1)];
CoverageOut=[LonOut LatOut];
%
%  the routine is done
%
return;
%
