function [TUVOut]=GetHFSpatialStats(TUVIn)
%
%  [TUVOut]=GetHFSpatialStats(TUVIn);
%
%  CMH, Nov 2007
%
%  Inputs:  TUVIn - Either a Tuv structure or a radial structure for a
%           single site (as found in Rcat).  Each site must be passed singly to this routine
%           if a radial structure is passed.  It is assumed that the passed
%           structure is organized from earliest time to latest time.
%
%  Outputs:  TUVOut - the output structure, spatial grid, etc., is the same as the
%  input structure.  The added fields are:
%  SpatialStats.SumDist3:  The sum of the distances to the 3 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumDist5:  The sum of the distances to the 5 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumDist10:  The sum of the distances to the 10 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.AvDist3:  The sum of distances divided by the number of
%      nearby measurements
%  SpatialStats.AvDist5:  The sum of the distances to the 5 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.AvDist10:  The sum of the distances to the 10 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumAngle1p5:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 1.5 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (20 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.SumAngle2p0:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 2.0 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (35 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.SumAngle5p0:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 5 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (50 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.AvAngle1p5:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvAngle2p0:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvAngle5p0:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvMagRatio1p5:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%  SpatialStats.AvMagRatio2p0:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%  SpatialStats.AvMagRatio5p0:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%
%  Set up to automatically work on a uniform grid.
%
SpeedThreshold1=20;
SpeedThreshold2=35;
SpeedThreshold3=50;
%
%  Set up output structure
%
TUVOut=TUVIn;
%
TUVOut.SpatialStats.SumDist3=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumDist5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumDist10=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist3=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist10=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle5p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle5p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio5p0=NaN*ones(size(TUVOut.U));
%
%  Step 1 - get distance between each grid point and every other.
%  Down rows is the index corresponding to lon lat from TUVin, across
%  columns is the distance to the corresponding gridpoints (i.e., row 1,
%  column 3 is the distance between location number 1 and location number
%  3.  Row 1 column 4 is the distance between location number 1 and
%  location number 4).  Distances in km.
%
disp('calculating grid distances')
%
[GridDistanceMatrix]=GetHFGridDistances(TUVIn);
%
%  Step 2
%
%  Get the total distance of the nearest 3 points with measurements (then
%  5, then 10)
%
disp('calculating distance to nearest 3 points')
[TUVOut.SpatialStats.SumDist3,TUVOut.SpatialStats.AvDist3]=GetSumNearestValidLocations(TUVIn,GridDistanceMatrix,3);
disp('calculating distance to nearest 5 points')
[TUVOut.SpatialStats.SumDist5,TUVOut.SpatialStats.AvDist5]=GetSumNearestValidLocations(TUVIn,GridDistanceMatrix,5);
disp('calculating distance to nearest 10 points')
[TUVOut.SpatialStats.SumDist10,TUVOut.SpatialStats.AvDist10]=GetSumNearestValidLocations(TUVIn,GridDistanceMatrix,10);
%
%  Step 3 - Get the sum of the angle differences for the appropriate
%  thresholded vectors
%
disp('calculating sum of angles to points 1.5 distances away')
[TUVOut.SpatialStats.SumAngle1p5,TUVOut.SpatialStats.AvAngle1p5]=GetSumAngleDiff(TUVIn,GridDistanceMatrix,SpeedThreshold1,1.5);
disp('calculating sum of angles to points 2.0 distances away')
[TUVOut.SpatialStats.SumAngle2p0,TUVOut.SpatialStats.AvAngle2p0]=GetSumAngleDiff(TUVIn,GridDistanceMatrix,SpeedThreshold2,2);
disp('calculating sum of angles to points 5 distances away')
[TUVOut.SpatialStats.SumAngle5p0,TUVOut.SpatialStats.AvAngle5p0]=GetSumAngleDiff(TUVIn,GridDistanceMatrix,SpeedThreshold3,5);
%
%  Step 4 - get the magnitude ratios
%
disp('calculating mag ratios to points 1.5 distances away')
[TUVOut.SpatialStats.AvMagRatio1p5]=GetMagRatios(TUVIn,GridDistanceMatrix,SpeedThreshold1,1.5);
disp('calculating mag ratios to points 2.0 distances away')
[TUVOut.SpatialStats.AvMagRatio2p0]=GetMagRatios(TUVIn,GridDistanceMatrix,SpeedThreshold2,2.0);
disp('calculating mag ratios to points 5 distances away')
[TUVOut.SpatialStats.AvMagRatio5p0]=GetMagRatios(TUVIn,GridDistanceMatrix,SpeedThreshold3,5);
%
%  The routine is done
%
return;
%
function [GridMatrixOut]=GetHFGridDistances(StructIn)
%
%  [GridMatrixOut]=GetHFGridDistances(StructIn);
%
%  The output will be of size  n rows by n columns, where n is the number
%  of locations.  rows are the locations, columns are the other locations
%  that we want the distances to.
%
[NumLoc,NumCol]=size(StructIn.LonLat);
GridMatrixOut=NaN*ones(NumLoc,NumLoc);
%
%  Loop through, easy and memory saving.
%
for jloc=1:NumLoc,
    [s,a12,a21] = m_idist(StructIn.LonLat(:,1),StructIn.LonLat(:,2),StructIn.LonLat(jloc,1),StructIn.LonLat(jloc,2));
    s=s/1000;
    s=s';
    GridMatrixOut(jloc,:)=s;
end
%
%  zero out the diagonal
%
temmat=diag(ones(1,NumLoc));
temmat=double(~temmat);  % diagonal now has zeros
temmark=find(temmat==0);
GridMatrixOut(temmark)=0;
clear temmat temmark
%
%  the routine is done
%
return;
%
function [SumDistanceOut,AvDistanceOut]=GetSumNearestValidLocations(StructIn,GridDistanceMatrix,NumClosestPoints)
%
%  [SumDistanceOut]=GetSumNearestValidLocations(StructIn,GridDistanceMatrix
%  ,NumClosestPoints);
%
%  Distance of closest points will be NaN if the measurement under
%  consideration is a NaN;
%
SumDistanceOut=NaN*ones(size(StructIn.U));
AvDistanceOut=NaN*ones(size(StructIn.U));
%
[NumLoc,NumTimes]=size(StructIn.U);
%
for jtime=1:NumTimes,
    NonNanIndex=find(~isnan(StructIn.U(:,jtime)));
    if numel(NonNanIndex)>0,
        PointsFound=1:numel(NonNanIndex);
        PointsFound=PointsFound';
        for jpoint=1:numel(NonNanIndex),
            OtherPointIndex=find(PointsFound~=jpoint);
            if numel(OtherPointIndex)>0,
                OtherNonNanPoints=NonNanIndex(OtherPointIndex);
                DistToOtherPoints=GridDistanceMatrix(NonNanIndex(jpoint),OtherNonNanPoints);
                DistToOtherPoints=sort(DistToOtherPoints,'ascend');
                if numel(DistToOtherPoints)>NumClosestPoints,
                    SumDistanceOut(NonNanIndex(jpoint),jtime)=sum(DistToOtherPoints(1:NumClosestPoints));
                    AvDistanceOut(NonNanIndex(jpoint),jtime)=(sum(DistToOtherPoints(1:NumClosestPoints)))/NumClosestPoints;
                else
                    SumDistanceOut(NonNanIndex(jpoint),jtime)=sum(DistToOtherPoints);
                    AvDistanceOut(NonNanIndex(jpoint),jtime)=sum(DistToOtherPoints)/numel(DistToOtherPoints);
                end
            end
        end
    end
end
%
%  the routine is done
%
return;
%
function [SumAngleOut,AvAngleOut]=GetSumAngleDiff(StructIn,GridDistanceMatrix,SpeedThreshold,DistanceMultFactor)
%
%  [SumAngleOut]=GetSumAngleDiff(StructIn,GridDistanceMatrix,SpeedThreshold
%  ,DistanceMultFactor);
%
%  DistanceMultFactor is how many multiples of the typical grid spacing to search over when calculating the 
%  angle differences
%
%  SumAngleOut will be NaNs at a given time and location if not enough
%  valid measurements found within the specified distance
%
SumAngleOut=NaN*ones(size(StructIn.U));
AvAngleOut=NaN*ones(size(StructIn.U));
%
[NumLoc,NumTimes]=size(StructIn.U);
%
GridTem=GridDistanceMatrix;
TemmarkGrid=find(GridTem==0);
if numel(TemmarkGrid)>0,
    GridTem(TemmarkGrid)=NaN;
end
TypMinDist=mode(nanmin(GridTem));
DistToUse=DistanceMultFactor*TypMinDist;
%
clear GridTem
%
for jtime=1:NumTimes,
    NonNanIndex=find(~isnan(StructIn.U(:,jtime)));
    if numel(NonNanIndex)>0,
        PointsFound=1:numel(NonNanIndex);
        PointsFound=PointsFound';
        for jpoint=1:numel(NonNanIndex),
            OtherPointIndex=find(PointsFound~=jpoint);
            if numel(OtherPointIndex)>0,
                OtherNonNanPoints=NonNanIndex(OtherPointIndex);
                DistToOtherPoints=GridDistanceMatrix(NonNanIndex(jpoint),OtherNonNanPoints);
                TemmarkDist=find(DistToOtherPoints<=DistToUse);
                if numel(TemmarkDist)>0,
                    OtherNonNanPoints=OtherNonNanPoints(TemmarkDist);
                    DistToOtherPoints=DistToOtherPoints(TemmarkDist);
                    OtherEastVel=StructIn.U(OtherNonNanPoints,jtime);
                    OtherNorthVel=StructIn.V(OtherNonNanPoints,jtime);
                    CurrentEastVel=StructIn.U(NonNanIndex(jpoint),jtime);
                    CurrentNorthVel=StructIn.V(NonNanIndex(jpoint),jtime);
                    [OtherMagOut,OtherDirOut]=ConvertComponentsToSpeedDir(OtherEastVel,OtherNorthVel);
                    [CurrentMagOut,CurrentDirOut]=ConvertComponentsToSpeedDir(CurrentEastVel,CurrentNorthVel);
                    AngDiff=OtherDirOut-CurrentDirOut;
                    Temmark1=find(AngDiff<-180);
                    Temmark2=find(AngDiff>180);
                    if numel(Temmark1)>0,
                        AngDiff(Temmark1)=360+AngDiff(Temmark1);
                    end
                    if numel(Temmark2)>0,
                        AngDiff(Temmark2)=360-AngDiff(Temmark2);
                    end
                    %
                    AngDiff=abs(AngDiff);
                    %
                    TemmarkThresh=find(OtherMagOut<=SpeedThreshold);
                    if numel(TemmarkThresh)>0,
                        if CurrentMagOut<=SpeedThreshold,
                            AngDiff(TemmarkThresh)=0;
                        end
                    end
                    %
                    SumAngleOut(NonNanIndex(jpoint),jtime)=sum(AngDiff);
                    AvAngleOut(NonNanIndex(jpoint),jtime)=sum(AngDiff)/numel(AngDiff);
                    %
                end
            end
        end
    end
end
%
%  the routine is done
%
return;
%
function [MagRatiosOut]=GetMagRatios(StructIn,GridDistanceMatrix,SpeedThreshold,DistanceMultFactor)
%
%[MagRatiosOut]=GetMagRatios(StructIn,GridDistanceMatrix,SpeedThreshold,DistanceMultFactor);
%
%  DistanceMultFactor is how many multiples of the typical grid spacing to search over when calculating the 
%  angle differences
%
%  MagRatiosOut will be NaNs at a given time and location if not enough
%  valid measurements found within the specified distance
%
MagRatiosOut=NaN*ones(size(StructIn.U));
%
[NumLoc,NumTimes]=size(StructIn.U);
%
GridTem=GridDistanceMatrix;
TemmarkGrid=find(GridTem==0);
if numel(TemmarkGrid)>0,
    GridTem(TemmarkGrid)=NaN;
end
TypMinDist=mode(nanmin(GridTem));
DistToUse=DistanceMultFactor*TypMinDist;
%
clear GridTem
%
for jtime=1:NumTimes,
    NonNanIndex=find(~isnan(StructIn.U(:,jtime)));
    if numel(NonNanIndex)>0,
        PointsFound=1:numel(NonNanIndex);
        PointsFound=PointsFound';
        for jpoint=1:numel(NonNanIndex),
            OtherPointIndex=find(PointsFound~=jpoint);
            if numel(OtherPointIndex)>0,
                OtherNonNanPoints=NonNanIndex(OtherPointIndex);
                DistToOtherPoints=GridDistanceMatrix(NonNanIndex(jpoint),OtherNonNanPoints);
                TemmarkDist=find(DistToOtherPoints<=DistToUse);
                if numel(TemmarkDist)>0,
                    OtherNonNanPoints=OtherNonNanPoints(TemmarkDist);
                    DistToOtherPoints=DistToOtherPoints(TemmarkDist);
                    OtherEastVel=StructIn.U(OtherNonNanPoints,jtime);
                    OtherNorthVel=StructIn.V(OtherNonNanPoints,jtime);
                    CurrentEastVel=StructIn.U(NonNanIndex(jpoint),jtime);
                    CurrentNorthVel=StructIn.V(NonNanIndex(jpoint),jtime);
                    OtherMagOut=abs(OtherEastVel+(i*OtherNorthVel));
                    CurrentMagOut=abs(CurrentEastVel+(i*CurrentNorthVel));
                    %
                    if CurrentMagOut<=SpeedThreshold,
                        TemmarkThresh=find(OtherMagOut>SpeedThreshold);
                        if numel(TemmarkThresh)>0,
                            OtherMagOut=OtherMagOut(TemmarkThresh);
                            MagRatiosOut(NonNanIndex(jpoint),jtime)=CurrentMagOut/mean(OtherMagOut);
                        else
                            MagRatiosOut(NonNanIndex(jpoint),jtime)=1;
                        end
                    else
                        MagRatiosOut(NonNanIndex(jpoint),jtime)=CurrentMagOut/mean(OtherMagOut);
                    end
                end
            end
        end
    end
end
%
%  the routine is done
%
return;


