function [TUVOut]=GetHFSpatialStatsSlow(TUVIn)
%
%  [TUVOut]=GetHFSpatialStatsSlow(TUVIn);
%
%  CMH, Sept 2008
%
%  based on GetHFSpatialStats.  Does by loops instead of matrices to save
%  memory when getting the closest points to calculate the statistics.  The
%  grid from Monterey to Arena is starting to get too big to use really
%  large spatially based matrices.
%
%  CMH, Nov 2007
%
%  Inputs:  TUVIn - Either a Tuv structure or a radial structure for a
%           single site (as found in Rcat).  Each site must be passed singly to this routine
%           if a radial structure is passed.  It is assumed that the passed
%           structure is organized from earliest time to latest time.
%
%  Outputs:  TUVOut - the output structure, spatial grid, etc., is the same as the
%  input structure.  The added fields are:
%  SpatialStats.SumDist3:  The sum of the distances to the 3 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumDist5:  The sum of the distances to the 5 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumDist10:  The sum of the distances to the 10 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.AvDist3:  The sum of distances divided by the number of
%      nearby measurements
%  SpatialStats.AvDist5:  The sum of the distances to the 5 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.AvDist10:  The sum of the distances to the 10 closest points
%      with valid current measurements (non-NaN) to the measurement under
%      consideration, in kilometers
%  SpatialStats.SumAngle1p5:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 1.5 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (20 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.SumAngle2p0:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 2.0 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (35 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.SumAngle5p0:  The sum of the absolute angle differences between the
%      measurement under consideration and points within 5 times the minimum
%      grid spacing away.  If the measurement under consideration and a
%      point nearby are both under a threshold (50 cm/s here), then the
%      difference is taken as zero.
%  SpatialStats.AvAngle1p5:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvAngle2p0:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvAngle5p0:  The average angle difference (from SumAngle1p5
%      and the number of points going into the total sum).
%  SpatialStats.AvMagRatio1p5:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%  SpatialStats.AvMagRatio2p0:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%  SpatialStats.AvMagRatio5p0:  The mag ratio between the vector under consideration
%      and the average mags of teh nearby vectors (vector / average of
%      others).
%
%  Set up to automatically work on a uniform grid.
%
SpeedThreshold1=20;
SpeedThreshold2=35;
SpeedThreshold3=50;
%
%  Set up output structure
%
TUVOut=TUVIn;
%
TUVOut.SpatialStats.SumDist3=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumDist5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumDist10=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist3=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvDist10=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.SumAngle5p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvAngle5p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio1p5=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio2p0=NaN*ones(size(TUVOut.U));
TUVOut.SpatialStats.AvMagRatio5p0=NaN*ones(size(TUVOut.U));
%
%  Step 1 - Let us loop through GridPoint by GridPoint and time by time - get indices of closest 10
%  points, then calculate the distance stats
%
[NumPoints,NumTimes]=size(TUVOut.U);
%
for jpoint=1:NumPoints,
    %
    for jtime=1:NumTimes,
        %
        if ~isnan(TUVOut.U(jpoint,jtime)), % if measurement is NaN, stat is NaN
            %
             [IndicesOfPoints]=GetClosestPointIndices(TUVOut,jpoint,jtime,30);  % 30 closest point indices (with valid measurements!
            %  otherwise stats turn out incorrectly
            %
            [TUVOut.SpatialStats.SumDist3(jpoint,jtime),TUVOut.SpatialStats.AvDist3(jpoint,jtime)]=GetSumNearestValidLocationsSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,3);
            %
            [TUVOut.SpatialStats.SumDist5(jpoint,jtime),TUVOut.SpatialStats.AvDist5(jpoint,jtime)]=GetSumNearestValidLocationsSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,5);
            %
            [TUVOut.SpatialStats.SumDist10(jpoint,jtime),TUVOut.SpatialStats.AvDist10(jpoint,jtime)]=GetSumNearestValidLocationsSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,10);
            %
        end
    end
    if isequal(rem(jpoint,50),0),
        TemStr=['Distance Stats: Point ' num2str(jpoint) ' of ' num2str(NumPoints)];
        disp(TemStr)
    end
end
%
%  Step 2 - Now get the minimum "typical" spacing for the grid
%
[MinTypSpacing]=GetTypGridSpacing(TUVIn.LonLat);
%
%
%  Step 3 - Let us loop through GridPoint by GridPoint and time by time -
%  get indices of closest points to calculate the angle stats
%
for jpoint=1:NumPoints,
    %
    [IndicesOfPoints]=GetPointsWithinDist(TUVIn.LonLat,jpoint,5*MinTypSpacing); % Indices
            %  of points within 5 times the spacing
    %
    for jtime=1:NumTimes,
        %
        if ~isnan(TUVOut.U(jpoint,jtime)), % if measurement is NaN, stat is NaN
            %
            [TUVOut.SpatialStats.SumAngle1p5(jpoint,jtime),TUVOut.SpatialStats.AvAngle1p5(jpoint,jtime)]=GetSumAngleDiffSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold1,1.5*MinTypSpacing);
            %
            [TUVOut.SpatialStats.SumAngle2p0(jpoint,jtime),TUVOut.SpatialStats.AvAngle2p0(jpoint,jtime)]=GetSumAngleDiffSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold2,2*MinTypSpacing);
            %
            [TUVOut.SpatialStats.SumAngle5p0(jpoint,jtime),TUVOut.SpatialStats.AvAngle5p0(jpoint,jtime)]=GetSumAngleDiffSlow...
                (TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold3,5*MinTypSpacing);
            %
            [TUVOut.SpatialStats.AvMagRatio1p5(jpoint,jtime)]=GetMagRatiosSlow(TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold1,1.5*MinTypSpacing);
            %
            [TUVOut.SpatialStats.AvMagRatio2p0(jpoint,jtime)]=GetMagRatiosSlow(TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold2,2*MinTypSpacing);
            %
            [TUVOut.SpatialStats.AvMagRatio5p0(jpoint,jtime)]=GetMagRatiosSlow(TUVIn,IndicesOfPoints,jpoint,jtime,SpeedThreshold3,5*MinTypSpacing);
            %
        end
    end
    if isequal(rem(jpoint,50),0),
        TemStr=['Angular Stats: Point ' num2str(jpoint) ' of ' num2str(NumPoints)];
        disp(TemStr)
    end
end
%
%  the routine is done
%
return;
%
function [IndicesOfPoints]=GetClosestPointIndices(StructIn,PointNumber,TimeNumber,NumberOfIndicesToGet)
%
%  [IndicesOfPoints]=GetClosestPointIndices(StructIn,PointNumber,NumberOfIndicesToGet);
%
% VelFieldIn - Number of points down rows, number of times across columns
%
%  This is not as slow as it could be - could use a loop and do point by
%  point
%
%  Does not account for the case where some of the distances are equal
%
[s,a12,a21] = m_idist(StructIn.LonLat(:,1),StructIn.LonLat(:,2),StructIn.LonLat(PointNumber,1),StructIn.LonLat(PointNumber,2));
s=s/1000;
s(PointNumber)=0;
%
[Snew,IndicesOfPoints]=sort(s,'ascend');
temmark=find(IndicesOfPoints~=PointNumber);
IndicesOfPoints=IndicesOfPoints(temmark);
%
MeasCheck=StructIn.U(IndicesOfPoints,TimeNumber);
temmark=find(~isnan(MeasCheck));
if numel(temmark)>0,
    IndicesOfPoints=IndicesOfPoints(temmark);
else
    IndicesOfPoints=NaN;
    return;
end
%
if numel(IndicesOfPoints)>=NumberOfIndicesToGet,
    IndicesOfPoints=IndicesOfPoints(1:NumberOfIndicesToGet);
end
%
%  the routine is done
%
return;
%
function [SumDistanceOut,AvDistanceOut]=GetSumNearestValidLocationsSlow(StructIn,IndicesOfPoints,PointNumber,TimeNumber,NumberOfClosestPointsToUse)
%
%  [SumDistanceOut,AvDistanceOut]=GetSumNearestValidLocationsSlow(StructIn,IndicesOfPoints,P
%  ointNumber,TimeNumber,NumberOfClosestPointsToUse);
%
%  IndicesOfPoints are the indices of the closest points
%
%  NumberOfClosestPointsToUse is how many points to use to calculate the statistic
%
%  Step 1 - Find the indices of the non-NaN points
%
if isequal(numel(IndicesOfPoints),1),
    if isnan(IndicesOfPoints),
        SumDistanceOut=NaN;
        AvDistanceOut=NaN;
        return;
    end
end
%
MeasOfInd=StructIn.U(IndicesOfPoints,TimeNumber);
temmark=find(~isnan(MeasOfInd));
if numel(temmark)>0,
    %
    IndicesOfPoints=IndicesOfPoints(temmark);
    %
else
    %
    %  no points found
    %
    SumDistanceOut=NaN;
    AvDistanceOut=NaN;
    return;
    %
end
%
%  Step 2 - calculate the distance from the point under consideration to the other
%  points
%
%try
   [s,a12,a21] = m_idist(StructIn.LonLat(IndicesOfPoints,1),StructIn.LonLat(IndicesOfPoints,2),StructIn.LonLat(PointNumber,1),StructIn.LonLat(PointNumber,2));
%catch
%    keyboard
%end
%
s=s/1000;
%
s=sort(s,'ascend');
%
if numel(s)>NumberOfClosestPointsToUse,
    SumDistanceOut=sum(s(1:NumberOfClosestPointsToUse));
    AvDistanceOut=(sum(s(1:NumberOfClosestPointsToUse)))/NumberOfClosestPointsToUse;
else
    SumDistanceOut=sum(s);
    AvDistanceOut=sum(s)/numel(s);
end
%
%  the routine is done
%
return;
%
function [MinTypSpacing]=GetTypGridSpacing(LonLatFieldIn)
%
%  [MinTypSpacing]=GetTypGridSpacing(LonLatFieldIn);
%
%  returns the "typical" grid spacing in km
%
%  does calculation based on up to 50 gridpoints.  Should be close enough
%
[NumGrids,NumCols]=size(LonLatFieldIn);
%
if NumGrids>50,
    %
    PointSpacing=floor(NumGrids/50);
    PointIndices=1:PointSpacing:NumGrids;
    PointIndices=PointIndices(1:50);
    %
else
    PointIndices=1:NumGrids;
end
%
MinDistVector=NaN*ones(1,numel(PointIndices));
%
for jgrid=1:NumGrids,
    %
    for jindex=1:numel(PointIndices),
        %
        if ~isequal(jgrid,PointIndices(jindex)),
            %
            [s,a12,a21] = m_idist(LonLatFieldIn(jgrid,1),LonLatFieldIn(jgrid,2),LonLatFieldIn(PointIndices(jindex),1),LonLatFieldIn(PointIndices(jindex),2));
            s=s/1000;
            %
            TemMin=nanmin([MinDistVector(jindex) s]);
            MinDistVector(jindex)=TemMin;
            %
        end
    end
end
%
%  now get the typical spacing
%
MinTypSpacing=mode(MinDistVector);
%
return;
%
function [IndicesOfPoints]=GetPointsWithinDist(LonLatFieldIn,PointNumber,SpacingInKm)
%
%  Get Indices of points within a certain distance of the selected point
%
%  routine is not as slow and memory saving as it could be
%
[s,a12,a21] = m_idist(LonLatFieldIn(:,1),LonLatFieldIn(:,2),LonLatFieldIn(PointNumber,1),LonLatFieldIn(PointNumber,2));
s=s/1000;
s(PointNumber)=0;
%
IndicesOfPoints=find(s<=SpacingInKm);
temmark=find(IndicesOfPoints~=PointNumber);
if numel(temmark)>0,
    IndicesOfPoints=IndicesOfPoints(temmark);
else
    IndicesOfPoints=NaN;
end
%
%  the routine is done
%
return;
%
function [SumAngle,AvAngle]=GetSumAngleDiffSlow(StructIn,IndicesOfPoints,PointNumber,TimeNumber,SpeedThreshold,DistanceToPoints)
%
%  [SumAngle,AvAngle]=GetSumAngleDiffSlow(StructIn,IndicesOfPoints,PointNum
%  ber,TimeNumber,SpeedThreshold,DistanceToPoints);
%
%
%  Step 1 - Find the indices of the non-NaN points
%
MeasOfInd=StructIn.U(IndicesOfPoints,TimeNumber);
temmark=find(~isnan(MeasOfInd));
if numel(temmark)>0,
    %
    IndicesOfPoints=IndicesOfPoints(temmark);
    %
else
    %
    %  no points found
    %
    SumAngle=NaN;
    AvAngle=NaN;
    return;
    %
end
%
%  Step 2 - find the non-NaN points within the correct distance
%
[s,a12,a21] = m_idist(StructIn.LonLat(IndicesOfPoints,1),StructIn.LonLat(IndicesOfPoints,2),StructIn.LonLat(PointNumber,1),StructIn.LonLat(PointNumber,2));
s=s/1000;
%
temmark=find(s<=DistanceToPoints);
if numel(temmark)>0,
    IndicesOfPoints=IndicesOfPoints(temmark);
else
    SumAngle=NaN;
    AvAngle=NaN;
    return;  % means no valid points found
end
%
[OtherMagOut,OtherDirOut]=ConvertComponentsToSpeedDir(StructIn.U(IndicesOfPoints,TimeNumber),StructIn.V(IndicesOfPoints,TimeNumber));
[CurrentMagOut,CurrentDirOut]=ConvertComponentsToSpeedDir(StructIn.U(PointNumber,TimeNumber),StructIn.V(PointNumber,TimeNumber));
AngDiff=OtherDirOut-CurrentDirOut;
Temmark1=find(AngDiff<-180);
Temmark2=find(AngDiff>180);
if numel(Temmark1)>0,
    AngDiff(Temmark1)=360+AngDiff(Temmark1);
end
if numel(Temmark2)>0,
    AngDiff(Temmark2)=360-AngDiff(Temmark2);
end
%
AngDiff=abs(AngDiff);
%
TemmarkThresh=find(OtherMagOut<=SpeedThreshold);
if numel(TemmarkThresh)>0,
    if CurrentMagOut<=SpeedThreshold,
        AngDiff(TemmarkThresh)=0;
    end
end
%
SumAngle=sum(AngDiff);
AvAngle=sum(AngDiff)/numel(AngDiff);
%
%  the routine is done
%
return;
%




function [MagRatioOut]=GetMagRatiosSlow(StructIn,IndicesOfPoints,PointNumber,TimeNumber,SpeedThreshold,DistanceToPoints)
%
%[MagRatioOut]=GetMagRatiosSlow(StructIn,IndicesOfPoints,PointNumber,TimeNumber,SpeedThreshold,DistanceToPoints);
%
%
%  MagRatiosOut will be NaNs at a given time and location if not enough
%  valid measurements found within the specified distance
%
%
%  Step 1 - Find the indices of the non-NaN points
%
MeasOfInd=StructIn.U(IndicesOfPoints,TimeNumber);
temmark=find(~isnan(MeasOfInd));
if numel(temmark)>0,
    %
    IndicesOfPoints=IndicesOfPoints(temmark);
    %
else
    %
    %  no points found
    %
    MagRatioOut=NaN;
    return;
    %
end
%
%  Step 2 - find the non-NaN points within the correct distance
%
[s,a12,a21] = m_idist(StructIn.LonLat(IndicesOfPoints,1),StructIn.LonLat(IndicesOfPoints,2),StructIn.LonLat(PointNumber,1),StructIn.LonLat(PointNumber,2));
s=s/1000;
%
temmark=find(s<=DistanceToPoints);
if numel(temmark)>0,
    IndicesOfPoints=IndicesOfPoints(temmark);
else
    MagRatioOut=NaN;
    return;  % means no valid points found
end
%
[OtherMagOut,OtherDirOut]=ConvertComponentsToSpeedDir(StructIn.U(IndicesOfPoints,TimeNumber),StructIn.V(IndicesOfPoints,TimeNumber));
[CurrentMagOut,CurrentDirOut]=ConvertComponentsToSpeedDir(StructIn.U(PointNumber,TimeNumber),StructIn.V(PointNumber,TimeNumber));
%
if CurrentMagOut<=SpeedThreshold,
    TemmarkThresh=find(OtherMagOut>SpeedThreshold);
    if numel(TemmarkThresh)>0,
        OtherMagOut=OtherMagOut(TemmarkThresh);
        MagRatioOut=CurrentMagOut/mean(OtherMagOut);
    else
        MagRatioOut=1;
    end
else
    MagRatioOut=CurrentMagOut/mean(OtherMagOut);
end
%
%  the routine is done
%
return;
%
