function [TUVOut]=GetHFVortDiv(TUVIn,GradientSwitch,KmWithinForCalc,Smooth1,Smooth2)
%
%  [TUVOut]=GetHFVortDiv(TUVStruct,GradientSwitch,KmWithinToCalc,Smoooth1,Smooth2);
%
%  CMH, Sept 2007
%
%  Inputs:  TUVIn - the TUV Structure to get the gradients for
%           Gradientwitch - a switch that can have one of several values:
%                           'dudx', 'dudy', 'dvdx', 'dvdy', 'vortcalc',
%                           'divcalc', 'all'
%           KmWithinForCalc - how close a point has to be the location in quuestion
%                            in order to calculate a gradient (5 km, means points 5 km 
%                            away will be used, so it's really vortcity over 10 km)
%           Smooth1 - optional input.  How much to smooth the current field before
%                     calculating the derivatives.  Default is unsmoothed.
%                     Smooth1 is just a number - for example, "3" means to
%                     smooth over 3 lat and lon points.  If zero, NaN, or
%                     unspecified, no smoothing is done.
%           Smooth2 - optional input.  How much to smooth the derived field after
%                     calculating the derivatives.  Default is unsmoothed.
%
%  Outputs:  TUVOut - the desired field in a TUV structure.
%
%  We have to loop, b/c we may have NaNs at each time step
%
%  Set up the output TUV structure
%
%  Step 1 - Check for smoothing
%
if exist('Smooth1','var'),
    if ~isequal(Smooth1,0)&&~isnan(Smooth1),
        TUVIn=SmoothTotalHFData(TUVIn,Smooth1,NaN);
    end
end
%
%  Step 2 - Set up initial (unsmoothed by Smooth2) output structures.  Simply add
%  fields to TUVIn at this point
%
[NumPos]=numel(TUVIn.LonLat(:,1));
NumTimes=numel(TUVIn.TimeStamp);
%
switch upper(GradientSwitch)
    case 'DUDX'
        TUVIn.dudx=NaN*ones(NumPos,NumTimes);
    case 'DUDY'
        TUVIn.dudy=NaN*ones(NumPos,NumTimes);
    case 'DVDX'
        TUVIn.dvdx=NaN*ones(NumPos,NumTimes);
    case 'DVDY'
        TUVIn.dvdy=NaN*ones(NumPos,NumTimes);
    case 'VORTCALC'
        TUVIn.vort=NaN*ones(NumPos,NumTimes);
    case 'DIVCALC'
        TUVIn.div=NaN*ones(NumPos,NumTimes);
    case 'ALL'
        TUVIn.dudx=NaN*ones(NumPos,NumTimes);
        TUVIn.dudy=NaN*ones(NumPos,NumTimes);
        TUVIn.dvdx=NaN*ones(NumPos,NumTimes);
        TUVIn.dvdy=NaN*ones(NumPos,NumTimes);
        TUVIn.vort=NaN*ones(NumPos,NumTimes);
        TUVIn.div=NaN*ones(NumPos,NumTimes);        
    otherwise
        %
        disp('incorrect value of GradientSwitch chosen')
        disp('routine GetHFVortDIV')
        disp('output set to NaN, routine terminating')
        TUVOut=NaN;
        return;
end
%
%  Step 3 - Loop through, calculate the requested gradients
%
for jtime=1:NumTimes,
    %
    RemTime=rem(jtime,10);
    if isequal(RemTime,0),
        disp('      ')
        disp('******')
        temstr=[num2str(jtime) ' time steps out of ' num2str(NumTimes) ' completed'];
        disp(temstr)
        temstr=[num2str(100*jtime/NumTimes) ' percent complete in calculation loop'];
        disp(temstr)
        disp('Routine GetHFVortDiv')
        disp('******')
        disp('      ')
    end
    %
    %  set up arrays for the indices of the closest points to the east, west, north, and south
    %
    InterpPoints.PointIndicesEast=GetClosestPointForGrad(TUVIn,jtime,'East');
    InterpPoints.PointIndicesWest=GetClosestPointForGrad(TUVIn,jtime,'West');
    InterpPoints.PointIndicesNorth=GetClosestPointForGrad(TUVIn,jtime,'North');
    InterpPoints.PointIndicesSouth=GetClosestPointForGrad(TUVIn,jtime,'South');
    %
    switch upper(GradientSwitch)
        case 'DUDX'
            TUVIn.dudx(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'DUDY'
            TUVIn.dudy(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'DVDX'
            TUVIn.dvdx(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'DVDY'
            TUVIn.dvdy(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'VORTCALC'
            TUVIn.vort(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'DIVCALC'
            TUVIn.div(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,GradientSwitch);
        case 'ALL'
            TUVIn.dudx(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'dudx');
            TUVIn.dudy(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'dudy');
            TUVIn.dvdx(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'dvdx');
            TUVIn.dvdy(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'dvdy');
            TUVIn.vort(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'vortcalc');
            TUVIn.div(:,jtime)=GetDerivativeQuantity(TUVIn,jtime,InterpPoints,KmWithinForCalc,'divcalc');
    end
end
%
%  Step 4 - Smooth the output, if requested
%
if exist('Smooth2','var'),
    if ~isequal(Smooth2,0)&&~isnan(Smooth2),
        switch upper(GradientSwitch)
            case 'DUDX'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dudx);
                TUVOut.dudx=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'DUDY'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dudy);
                TUVOut.dudy=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'DVDX'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dvdx);
                TUVOut.dvdx=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'DVDY'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dvdy);
                TUVOut.dvdy=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'VORTCALC'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.vort);
                TUVOut.vort=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'DIVCALC'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.div);
                TUVOut.div=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
            case 'ALL'
                TUVOut=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dudx);
                TUVOut.dudx=TUVOut.QuantSmoothed;
                TUVOut=rmfield(TUVOut,'QuantSmoothed');
                %
                TUVOut2=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dudy);
                TUVOut.dudy=TUVOut2.QuantSmoothed;
                clear TUVOut2
                %
                TUVOut2=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dvdx);
                TUVOut.dvdx=TUVOut2.QuantSmoothed;
                clear TUVOut2
                %
                TUVOut2=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.dvdy);
                TUVOut.dvdy=TUVOut2.QuantSmoothed;
                clear TUVOut2
                %
                TUVOut2=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.vort);
                TUVOut.vort=TUVOut2.QuantSmoothed;
                clear TUVOut2
                %
                TUVOut2=SmoothTotalHFData(TUVIn,Smooth2,NaN,TUVIn.div);
                TUVOut.div=TUVOut2.QuantSmoothed;
                clear TUVOut2
                %
        end
    else
        TUVOut=TUVIn;
    end
else
    TUVOut=TUVIn;
end
%
%  the routine is done
%
return;
%
function [PtIndex]=GetClosestPointForGrad(T,TimeStep,DirectionSwitch)
%
%  [PtIndex]=GetClosestPointForGrad(T,TimeStep,DirectionSwitch);
%
%  like GetClosestPointForVel, except is does not allow for the possibility that the points returned can
%  be equal to the input point.  We want to interpolate over a wider range than that.
%  There must be some separation between them.
%
%  Assumes that the sites are east or west (west negative)
%
%  Gets closest point to the 
%  individual grids of T, unless the closest velocities are NaN,
%  in which case it gets the next closest
%  point.
%
%  assumes a fairly uniform (rectangular) grid!!!!
%
PtIndex=NaN*ones(numel(T.LonLat(:,1)),1);
%
DegCutOff=0.005;  %  will find points within 0.005 degrees to see if they are on a straight line
%
for jPt=1:numel(PtIndex),
    %
    switch upper(DirectionSwitch)
        case 'NORTH'
            Temmark=find(abs(T.LonLat(:,1)-T.LonLat(jPt,1))<=DegCutOff);  % points on the same line
            if numel(Temmark)>0,
                TemmarkPtNew=find(T.LonLat(Temmark,2)>=(T.LonLat(jPt,2)+DegCutOff));
                if numel(TemmarkPtNew)>0,
                    TemmarkPt=Temmark(TemmarkPtNew);
                else
                    TemmarkPt=NaN;
                end
            else
                TemmarkPt=NaN;
            end
        case 'SOUTH'
            Temmark=find(abs(T.LonLat(:,1)-T.LonLat(jPt,1))<=DegCutOff);  % points on the same line
            if numel(Temmark)>0,
                TemmarkPtNew=find(T.LonLat(Temmark,2)<=(T.LonLat(jPt,2)-DegCutOff));
                if numel(TemmarkPtNew)>0,
                    TemmarkPt=Temmark(TemmarkPtNew);
                else
                    TemmarkPt=NaN;
                end
            else
                TemmarkPt=NaN;
            end
        case 'EAST'
            Temmark=find(abs(T.LonLat(:,2)-T.LonLat(jPt,2))<=DegCutOff);  % points on the same line
            if numel(Temmark)>0,
                TemmarkPtNew=find(T.LonLat(Temmark,1)>=(T.LonLat(jPt,1)+DegCutOff));
                if numel(TemmarkPtNew)>0,
                    TemmarkPt=Temmark(TemmarkPtNew);
                else
                    TemmarkPt=NaN;
                end
            else
                TemmarkPt=NaN;
            end
        case 'WEST'
            Temmark=find(abs(T.LonLat(:,2)-T.LonLat(jPt,2))<=DegCutOff);  % points on the same line
            if numel(Temmark)>0,
                TemmarkPtNew=find(T.LonLat(Temmark,1)<=(T.LonLat(jPt,1)-DegCutOff));
                if numel(TemmarkPtNew)>0,
                    TemmarkPt=Temmark(TemmarkPtNew);
                else
                    TemmarkPt=NaN;
                end
            else
                TemmarkPt=NaN;
            end
    end
    %
    %  we have the indices of possibly acceptable points.  Now calculate
    %  the index of the nearest point with valid data.
    %
    if numel(TemmarkPt>=1)&&~isnan(TemmarkPt(1)),
        %
        TemLon=T.LonLat(jPt,1)*ones(numel(TemmarkPt),1);
        TemLat=T.LonLat(jPt,2)*ones(numel(TemmarkPt),1);
        [TemDist,a12,a21] = m_idist(TemLon,TemLat,T.LonLat(TemmarkPt,1),T.LonLat(TemmarkPt,2),'wgs84');
        %
        DataCheck=T.U(TemmarkPt,TimeStep)+T.V(TemmarkPt,TimeStep);
        %
        [PtDist,IndDist]=sort(TemDist,'ascend');
        DataCheck=DataCheck(IndDist);
        TemmarkPt=TemmarkPt(IndDist);  % now from closest to farthest away
        %
        DataCheck=DataCheck.*PtDist;  % now Non-NaN where valid data, arranged from closest to farthest point
        %
        TemmarkDist=find(~isnan(DataCheck));
        if numel(TemmarkDist)>0,
            PtIndex(jPt)=TemmarkPt(TemmarkDist(1));
        end
    end
end
%
%  the routine is done
%
return;
%
function [SeriesOut]=GetDerivativeQuantity(TUVIn,TimeIndexIn,InterpPoints,KmWithinForCalc,GradientSwitch)
%
%  [SeriesOut]=GetDerivativeQuantity(TUVIn,TimeIndexIn,InterpPoints,KmWithi
%  nForCalc,GradientSwitch);
%
%  assumes vel are in cm/s
%
%  SeriesOut will be a vector, rows equal to number of locations
%
%  in this routine, GradientSwitch can only have one value ('all' is no ta
%  valid option)
%
NumLocations=numel(TUVIn.LonLat(:,1));
SeriesOut=NaN*ones(NumLocations,1);
%
for jloc=1:NumLocations,
    %
    %  get East West Distance Axis
    %
    %  the difficulty is that we may get NaNs for the indices
    %
    if ~isnan(InterpPoints.PointIndicesWest(jloc)),
        WestLoc=TUVIn.LonLat(InterpPoints.PointIndicesWest(jloc),1);
        TemLatAxis=[TUVIn.LonLat(jloc,2)];
        TemLonAxis=[TUVIn.LonLat(jloc,1)];
        [TemDist,a12,a21] = m_idist(WestLoc,TemLatAxis,TemLonAxis,TemLatAxis,'wgs84');
        WestLoc=-1*TemDist;
        UWest=TUVIn.U(InterpPoints.PointIndicesWest(jloc),TimeIndexIn); % the measurements
        VWest=TUVIn.V(InterpPoints.PointIndicesWest(jloc),TimeIndexIn);
    else
        WestLoc=NaN;
        UWest=NaN;
        VWest=NaN;
    end
    %
    if ~isnan(InterpPoints.PointIndicesEast(jloc)),
        EastLoc=TUVIn.LonLat(InterpPoints.PointIndicesEast(jloc),1);
        TemLatAxis=[TUVIn.LonLat(jloc,2)];
        TemLonAxis=[TUVIn.LonLat(jloc,1)];
        [TemDist,a12,a21] = m_idist(EastLoc,TemLatAxis,TemLonAxis,TemLatAxis,'wgs84');
        EastLoc=TemDist;
        UEast=TUVIn.U(InterpPoints.PointIndicesEast(jloc),TimeIndexIn); % the measurements
        VEast=TUVIn.V(InterpPoints.PointIndicesEast(jloc),TimeIndexIn);
    else
        EastLoc=NaN;
        UEast=NaN;
        VEast=NaN;
    end
    %
    %  distances are in meters.  We should only keep those measurements
    %  that are less than or equal to KmWithinForCalc
    %
    EastWestAxis=[WestLoc 0 EastLoc];  % measured in meters from centerpoint
    EastWestUVel=[UWest TUVIn.U(jloc,TimeIndexIn) UEast];
    EastWestVVel=[VWest TUVIn.V(jloc,TimeIndexIn) VEast];
    %
    TemKm=find(abs(EastWestAxis)<=(1000*KmWithinForCalc));  % EastWestAxis is in meters
    if numel(TemKm)>0,
        EastWestAxis=EastWestAxis(TemKm);
        EastWestUVel=EastWestUVel(TemKm);
        EastWestVVel=EastWestVVel(TemKm);
    else
        EastWestAxis=NaN;
        EastWestUVel=NaN;
        EastWestVVel=NaN;
    end
    %
    %  now get north / south axis
    %
    if ~isnan(InterpPoints.PointIndicesNorth(jloc)),
        NorthLoc=TUVIn.LonLat(InterpPoints.PointIndicesNorth(jloc),2);
        TemLatAxis=[TUVIn.LonLat(jloc,2)];
        TemLonAxis=[TUVIn.LonLat(jloc,1)];
        [TemDist,a12,a21] = m_idist(TemLonAxis,NorthLoc,TemLonAxis,TemLatAxis,'wgs84');
        %TemDist=(1/1000)*TemDist;  % now in meters.  TemDist always positive
        NorthLoc=TemDist;
        UNorth=TUVIn.U(InterpPoints.PointIndicesNorth(jloc),TimeIndexIn); % the measurements
        VNorth=TUVIn.V(InterpPoints.PointIndicesNorth(jloc),TimeIndexIn);
    else
        NorthLoc=NaN;
        UNorth=NaN;
        VNorth=NaN;
    end
    %
    if ~isnan(InterpPoints.PointIndicesSouth(jloc)),
        SouthLoc=TUVIn.LonLat(InterpPoints.PointIndicesSouth(jloc),2);
        TemLatAxis=[TUVIn.LonLat(jloc,2)];
        TemLonAxis=[TUVIn.LonLat(jloc,1)];
        [TemDist,a12,a21] = m_idist(TemLonAxis,SouthLoc,TemLonAxis,TemLatAxis,'wgs84');
        %TemDist=(1/1000)*TemDist;  % now in meters.  TemDist always positive
        SouthLoc=-1*TemDist;
        USouth=TUVIn.U(InterpPoints.PointIndicesSouth(jloc),TimeIndexIn); % the measurements
        VSouth=TUVIn.V(InterpPoints.PointIndicesSouth(jloc),TimeIndexIn);
    else
        SouthLoc=NaN;
        USouth=NaN;
        VSouth=NaN;
    end
    %
    NorthSouthAxis=[SouthLoc 0 NorthLoc];
    NorthSouthUVel=[USouth TUVIn.U(jloc,TimeIndexIn) UNorth];
    NorthSouthVVel=[VSouth TUVIn.V(jloc,TimeIndexIn) VNorth];
    %
    TemKm=find(abs(NorthSouthAxis)<=(1000*KmWithinForCalc));  % NorthSouth Axis is in meters
    if numel(TemKm)>0,
        NorthSouthAxis=NorthSouthAxis(TemKm);
        NorthSouthUVel=NorthSouthUVel(TemKm);
        NorthSouthVVel=NorthSouthVVel(TemKm);
    else
        NorthSouthAxis=NaN;
        NorthSouthUVel=NaN;
        NorthSouthVVel=NaN;
    end
    %
    %  We now have distance axes and measurements where defined, NaNs where
    %  not defined
    %
    %  Obtain the east/west gradients
    %
    IndToUseUVel=find(~isnan(EastWestAxis)&~isnan(EastWestUVel));  % must have more than one measurement to calculate
    %  a gradient
    IndToUseVVel=find(~isnan(EastWestAxis)&~isnan(EastWestVVel));
    %
    if numel(IndToUseUVel)>=2,
        p=polyfit(EastWestAxis(IndToUseUVel),0.01*EastWestUVel(IndToUseUVel),1);  % 0.01 to convert to m/s
        dudx=p(1);
    else
        dudx=NaN;
    end
    %
    if numel(IndToUseVVel)>=2,
        p=polyfit(EastWestAxis(IndToUseUVel),0.01*EastWestVVel(IndToUseUVel),1);  % 0.01 to convert to m/s
        dvdx=p(1);
    else
        dvdx=NaN;
    end
    %
    %
    %  Obtain the north/south gradients
    %
    IndToUseUVel=find(~isnan(NorthSouthAxis)&~isnan(NorthSouthUVel));  % must have more than one measurement to calculate
    %  a gradient
    IndToUseVVel=find(~isnan(NorthSouthAxis)&~isnan(NorthSouthVVel));
    %
    if numel(IndToUseUVel)>=2,
        p=polyfit(NorthSouthAxis(IndToUseUVel),0.01*NorthSouthUVel(IndToUseUVel),1);  % 0.01 to convert to m/s
        dudy=p(1);
    else
        dudy=NaN;
    end
    %
    if numel(IndToUseVVel)>=2,
        p=polyfit(NorthSouthAxis(IndToUseUVel),0.01*NorthSouthVVel(IndToUseUVel),1);  % 0.01 to convert to m/s
        dvdy=p(1);
    else
        dvdy=NaN;
    end
    %
    %  now calculate the appropriate output
    %
    switch upper(GradientSwitch)
        %
        case 'DUDX'
            SeriesOut(jloc)=dudx;
        case 'DUDY'
            SeriesOut(jloc)=dudy;
        case 'DVDX'
            SeriesOut(jloc)=dvdx;
        case 'DVDY'
            SeriesOut(jloc)=dvdy;
        case 'VORTCALC'
            SeriesOut(jloc)=dvdx-dudy;
        case 'DIVCALC'
            SeriesOut(jloc)=dudx+dvdy;
    end
    %
end
%
%  the routine is done
%
return;
%

    
    
    


    

