function [TUVNew]=SmoothTotalHFData(TUVIn,SmoothPoints,NaNHandleSwitch,QuantToSmooth)
%
%  [TUVNew]=SmoothTotalHFData(TUVIn,SmoothPoints,NaNHandleSwitch,QuantToSmooth);
%
%  To smooth the totals data over a number of points equal to 
%  "SmoothPoints" points.  Number of points is assumed to be equal in the x
%  and y directions.
%
%  Note:  Smoothing just done directly on the lon and lat matrices.  Not
%  converted to meters and back to degrees.
%
%  If QuantToSmooth is passed, that quantity is smoothed and returned in
%  a QuantSmoothed field of the TUV structure.  LonLatSmoothed field is
%  also defined.
%
%  numcolumns in QuantToSmooth should equal the number of times in the
%  TUVIn structure
%
%  If QuantToSmooth is not passed, then U and V are smoothed, the output
%  structure has LonLat smoothed, and U and V are smoothed (the input
%  fields are replaced).  (U and V smothed anyway, by default.
%  QuantToSmooth is just additional)
%
%  NaNHandleSwitch not implemented yet.  Set to any value you'd like.
%
%  Step 1 - find the number of output locations, set up the structure
%  Using an array of ones to find the locations that correspond to what the
%  smoothing will be.  NaNs will occur where there is invalid data
%
%  Tested by simply making plotrs at various timesteps, not actual
%  calculation.  Seems to work.
%
[X,Y,V]=meshgrid_vector_data(TUVIn.LonLat(:,1),TUVIn.LonLat(:,2),ones(size(TUVIn.LonLat(:,1))));
SmoothFilt=(1/(SmoothPoints*SmoothPoints))*ones(SmoothPoints,SmoothPoints);
XSmooth=conv2(X,SmoothFilt,'valid');
YSmooth=conv2(Y,SmoothFilt','valid');
VSmooth=conv2(V,SmoothFilt','valid');
XSmooth=reshape(XSmooth,numel(XSmooth),1);
YSmooth=reshape(YSmooth,numel(YSmooth),1);
VSmooth=reshape(VSmooth,numel(VSmooth),1);
ValidIndices=find(~isnan(VSmooth));  % locations where we will have valid smoothed data
NumValidLocations=numel(ValidIndices);
NumTimes=numel(TUVIn.TimeStamp);
TUVNew=TUVstruct([NumValidLocations NumTimes], 1 );
TUVNew.LonLat=[XSmooth(ValidIndices) YSmooth(ValidIndices)];
TUVNew.TimeStamp=TUVIn.TimeStamp;
%
%  Step 2- check and make sure the dimensions of QuantToSmooth match up, if
%  specified.  If not, set up QuantToSmooth
%
if exist('QuantToSmooth','var'),
    [NumPosQuant,NumTimesQuant]=size(QuantToSmooth);
    if ~isequal(NumTimesQuant,NumTimes)||~isequal(NumPosQuant,numel(TUVIn.LonLat(:,1))),
        disp('the dimensions of the quantity to smooth do not match the input')
        disp('TUV dimensions.  NaN being returned.  Routine SmoothTotalHFData.')
        TUVNew=NaN;
        return;
    end
    %
    TUVNew.QuantSmoothed=NaN*ones(NumValidLocations,NumTimes);  % if it exists, set tthe structure up
    %
end
%
%  Step 3 - loop through, calculate the smoothed field
%
for jtime=1:numel(TUVIn.TimeStamp),
    %
    VectToSmooth=TUVIn.U(:,jtime)+(i*TUVIn.V(:,jtime));
    [X,Y,V] = meshgrid_vector_data(TUVIn.LonLat(:,1),TUVIn.LonLat(:,2),VectToSmooth);
    VSmooth=conv2(V,SmoothFilt','valid');
    VSmooth=reshape(VSmooth,numel(VSmooth),1);
    TUVNew.U(:,jtime)=real(VSmooth(ValidIndices));
    TUVNew.V(:,jtime)=imag(VSmooth(ValidIndices));
    %
    %  now smooth QuantToSmooth, if specified
    %
    if exist('QuantToSmooth','var'),
        VectToSmooth=QuantToSmooth(:,jtime);
        [X,Y,V] = meshgrid_vector_data(TUVIn.LonLat(:,1),TUVIn.LonLat(:,2),VectToSmooth);
        VSmooth=conv2(V,SmoothFilt','valid');
        VSmooth=reshape(VSmooth,numel(VSmooth),1);
        TUVNew.QuantSmoothed(:,jtime)=VSmooth(ValidIndices);
    end
    %
end
%
%  the routine is done
%
return;
%
