function [VelStructOut]=extractVelocities(Tuv,LonLatPositions,GridSlop,InterpSwitch)
%
%  [VelStructOut]=extractVelocities(Tuv,LonLatPositions,GridSlop,InterpSwitch);
%
%  To get velocities at selected lat / lon points from a Tuv structure (or any structure that
%  is organized as a Tuv structure, with timestamp fields, lat/lon positions, and U,V fields
%
%  Required Inputs:  
%
%  Tuv - the Tuv structure
%  LonLatPositions - a 2 column vector of positions to extract the velocities for
%     lon first column, lat second column
%
%  Optional Inputs:
%  GridSlop - Some of the desired positions may be out of the range of the valid velocities.  GridSlop is
%             how close (in km) a requested lon/lat position needs to be to the valid measurements to be 
%             considered the same.  For example, if GridSlop = 2, then a desired position can be 2 km
%             beyond the max range of the sonar and a velocity will still be returned.  Defaults to 2 km.
%  InterpSwitch - either 'closest' or 'interp'.....  closest simply returns the velocity at the closest gridpoint,
%             interp does a 2d interpolation of the 4  (or 3) surrounding velocity gridpoints.  Default is interp.
%             If the closest gridpoints have NaN velocities, then the next closest will be used.  If those also 
%             have NaN velocities, then the points will be out of range.  (and GridSlop will be used).
%
%  Outputs - the Velocity info in a Tuv type structure
%
%  uses m_idist to get distances, and wgs84 geoid (m_map toolbox must be installed)
%
%  longitudes and latitudes assumed to be smoothly increasing / decreasing (i.e., don't go from -180 longitude to
%  to 180 in the Tuv structure).
% 
%  modified 22 Sept 2010 to get divergence and vorticity (assumes fields
%  are named div and vort)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 	$Id: extractVelocities.m 2007-08 cmh $	
%
% Copyright (C) 2007 Christopher M. Halle
% Licence: GPL (Gnu Public License)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  check for optional inputs (GridSlop and InterpSwitch)
%
if ~exist('GridSlop','var'),
    GridSlop=2;
end
%
if ~exist('InterpSwitch','var'),
    InterpSwitch='interp';
end
%
%  let us loop through.  Slow, but o.k.  Can vectorize later if need be.
%
%  do a double loop!  (grr...) since the velocities may or may not be present at the different times,
%  and can affect our point selection
%
NumTimes=numel(Tuv.TimeStamp);
NumVelsPerTime=numel(LonLatPositions(:,1));
VelsFoundU=NaN*ones(NumVelsPerTime,NumTimes);
VelsFoundV=NaN*ones(NumVelsPerTime,NumTimes);
%
if isfield(Tuv,'vort'),
    VortFound=NaN*ones(NumVelsPerTime,NumTimes);
end
%
if isfield(Tuv,'div'),
    DivFound=NaN*ones(NumVelsPerTime,NumTimes);
end
%
for jvel=1:NumVelsPerTime,
    %
    if isequal(rem(jvel,20),0),
        TemStr=['Velocity Set Number ' num2str(jvel) ' of ' num2str(NumVelsPerTime)];
        disp(TemStr)
    end
    %
    for jtime=1:NumTimes,
        %
        %  Find the closest point (if any!) with lon >=desired lon, same for lat
        %
        QuadrantSwitch=1;
        Pt1Index=GetClosestPointForVel(Tuv,LonLatPositions(jvel,:),jtime,QuadrantSwitch);
        %
        %  Find the closest point (if any!) with lon < desired lon, lat >= lat desired
        %  (traveling around the point in a CCW fashion)
        %
        QuadrantSwitch=2;
        Pt2Index=GetClosestPointForVel(Tuv,LonLatPositions(jvel,:),jtime,QuadrantSwitch);
        %
        %
        %  Find the closest point (if any!) with lon < desired lon, lat < lat desired
        %  (traveling around the point in a CCW fashion)
        %
        QuadrantSwitch=3;
        Pt3Index=GetClosestPointForVel(Tuv,LonLatPositions(jvel,:),jtime,QuadrantSwitch);
        %
        %  Find the closest point (if any!) with lon >= desired lon, lat < lat desired
        %  (traveling around the point in a CCW fashion)
        %
        QuadrantSwitch=4;
        Pt4Index=GetClosestPointForVel(Tuv,LonLatPositions(jvel,:),jtime,QuadrantSwitch);
        %
        %
        %  if we have found 3 or 4 points, determine if the desired point is within the polygon set up
        %  by the points (there is also a "pathological" case, where the point is exactly on the edge of the domain,
        %  and exactly between 2 points.  This case not checked for here).
        %
        AllPointIndices=[Pt1Index;Pt2Index;Pt3Index;Pt4Index];
        TemmarkIndex=find(~isnan(AllPointIndices));
        AllPointIndices=AllPointIndices(TemmarkIndex); %assume some points were found!
        if numel(TemmarkIndex)>=3,
            %
            TemLon=[Tuv.LonLat(AllPointIndices,1);Tuv.LonLat(AllPointIndices(1),1)];
            TemLat=[Tuv.LonLat(AllPointIndices,2);Tuv.LonLat(AllPointIndices(1),2)];
            index = inpolygon(LonLatPositions(jvel,1),LonLatPositions(jvel,2),TemLon,TemLat);
            if index,
               PositionInPolygon='YES';
            else
                PositionInPolygon='NO';
            end
        else
            PositionInPolygon='NO';
        end
        %
        %  We now have 3 cases for this particular location and timestamp.  The point is in the polygon,
        %  and interp is desired, the point is in the polygon and closest is desired, or the point is out of the 
        %  polygon (or the nearest points did not have valid velocity measurements).
        %
        switch PositionInPolygon
            case 'YES'
                if isequal(upper(InterpSwitch),'INTERP'),
                    %
                    %  we have positions in terms of lon and lat.  We
                    %  should get positions in terms of meters or
                    %  kilometers.
                    %
                    %  get the values by a weighted average of the distance
                    %  from the desired point from the points found.  Saves
                    %  us trying to fit weird "planar", curved functions
                    %  if 4 vertex points are found.
                    %
                    LonToUse=Tuv.LonLat(AllPointIndices,1);
                    LatToUse=Tuv.LonLat(AllPointIndices,2);
                    UToUse=Tuv.U(AllPointIndices,jtime);
                    VToUse=Tuv.V(AllPointIndices,jtime);
                    %
                    TemLon=LonLatPositions(jvel,1)*ones(size(LonToUse));
                    TemLat=LonLatPositions(jvel,2)*ones(size(LonToUse));
                    %
                    [TemDist,a12,a21] = m_idist(TemLon,TemLat,LonToUse,LatToUse,'wgs84');
                    %
                    TotDist=sum(TemDist);
                    %
                    WtFactor=TotDist*ones(size(TemDist))-TemDist;  % velocities should be weighted by how close they
                    %  are to the desired point, TemDist is a measure of
                    %  how far away the desired point is
                    %
                    DenFactor=sum(WtFactor);
                    %
                    WtFactor=(1/DenFactor)*WtFactor;
                    %
                    TemU=sum(WtFactor.*UToUse);
                    TemV=sum(WtFactor.*VToUse);
                    %
                    VelsFoundU(jvel,jtime)=TemU;
                    VelsFoundV(jvel,jtime)=TemV;
                    %
                    %
                    if isfield(Tuv,'vort'),
                        %
                        VortToUse=Tuv.vort(AllPointIndices,jtime);
                        DivToUse=Tuv.div(AllPointIndices,jtime);
                        %
                        TemDiv=sum(WtFactor.*DivToUse);
                        TemVort=sum(WtFactor.*VortToUse);
                        %
                        %
                        VortFound(jvel,jtime)=TemVort;
                        DivFound(jvel,jtime)=TemDiv;
                        %
                    end
                    %                    
                else
                    % get closest points of the points found
                    TemLon=LonLatPositions(jvel,1)*ones(numel(AllPointIndices),1);
                    TemLat=LonLatPositions(jvel,2)*ones(numel(AllPointIndices),1);
                    [TemDist,a12,a21] = m_idist(TemLon,TemLat,Tuv.LonLat(AllPointIndices,1),Tuv.LonLat(AllPointIndices,2),'wgs84');
                    [MinDist,MinPos]=min(abs(TemDist));
                    ClosestIndex=AllPointIndices(MinPos);
                    VelsFoundU(jvel,jtime)=Tuv.U(ClosestIndex,jtime);
                    VelsFoundV(jvel,jtime)=Tuv.V(ClosestIndex,jtime);
                    %
                    if isfield(Tuv,'vort'),
                        VortFound(jvel,jtime)=Tuv.vort(ClosestIndex,jtime);
                        DivFound(jvel,jtime)=Tuv.div(ClosestIndex,jtime);
                    end
                    %
                end
            case 'NO'
                %
                %  we need to get points that are not NaNs, then search
                %  over those
                %
                TemIndNotNan=find((~isnan(Tuv.U(:,jtime)))&(~isnan(Tuv.V(:,jtime))));
                %
                if numel(TemIndNotNan>=1),
                    UToCheck=Tuv.U(TemIndNotNan,jtime);
                    VToCheck=Tuv.V(TemIndNotNan,jtime);
                    NumTPoints=numel(TemIndNotNan);
                    TemLon=LonLatPositions(jvel,1)*ones(NumTPoints,1);
                    TemLat=LonLatPositions(jvel,2)*ones(NumTPoints,1);
                    [TemDist,a12,a21] = m_idist(TemLon,TemLat,Tuv.LonLat(TemIndNotNan,1),Tuv.LonLat(TemIndNotNan,2),'wgs84');
                    [MinDist,MinPos]=min(abs(TemDist));
                    MinDist=MinDist/1000;  % convert to km
                    if MinDist<=GridSlop,
                        VelsFoundU(jvel,jtime)=UToCheck(MinPos);
                        VelsFoundV(jvel,jtime)=VToCheck(MinPos);
                    end
                end
                %
                %  added div and vort
                %
                if isfield(Tuv,'vort'),
                    TemIndNotNan=find((~isnan(Tuv.vort(:,jtime)))&(~isnan(Tuv.div(:,jtime))));
                    %
                    if numel(TemIndNotNan>=1),
                        VortToCheck=Tuv.vort(TemIndNotNan,jtime);
                        DivToCheck=Tuv.div(TemIndNotNan,jtime);
                        NumTPoints=numel(TemIndNotNan);
                        TemLon=LonLatPositions(jvel,1)*ones(NumTPoints,1);
                        TemLat=LonLatPositions(jvel,2)*ones(NumTPoints,1);
                        [TemDist,a12,a21] = m_idist(TemLon,TemLat,Tuv.LonLat(TemIndNotNan,1),Tuv.LonLat(TemIndNotNan,2),'wgs84');
                        [MinDist,MinPos]=min(abs(TemDist));
                        MinDist=MinDist/1000;  % convert to km
                        if MinDist<=GridSlop,
                            VortFound(jvel,jtime)=VortToCheck(MinPos);
                            DivFound(jvel,jtime)=DivToCheck(MinPos);
                        end
                    end
                end
        end
        %
        %  we now have interpolated one point at one time.  Repeat the bloody mess.
        %
    end
end
%
%  Set up an output structure  - use a Tuv structure just for kicks
%
VelStructOut=TUVstruct([numel(LonLatPositions(:,1)) NumTimes], 1 );
%
VelStructOut.LonLat=LonLatPositions;
VelStructOut.U=VelsFoundU;
VelStructOut.V=VelsFoundV;
VelStructOut.TimeStamp=Tuv.TimeStamp;
%
if isfield(Tuv,'vort'),
    VelStructOut.vort=VortFound;
    VelStructOut.div=DivFound;
end
%   
%  routine is done
%
return;
%
function [PtIndex]=GetClosestPointForVel(T,LonLatIn,TimeStep,QuadrantSwitch)
%
%  [PtIndex]=GetClosestPointForVel(T,LonLatIn,TimeStep,QuadrantSwitch):
%
%  Gets closest point to the 
%  one requessted from the grids of T, unless the closest velocities are NaN,
%  point is NaN, in which case it gets the next closest
%  point.
%
switch QuadrantSwitch
    case 1
        TemmarkPt=find((T.LonLat(:,1)>=LonLatIn(1))&(T.LonLat(:,2)>=LonLatIn(2)));
    case 2
        TemmarkPt=find((T.LonLat(:,1)<LonLatIn(1))&(T.LonLat(:,2)>=LonLatIn(2)));
    case 3
        TemmarkPt=find((T.LonLat(:,1)<LonLatIn(1))&(T.LonLat(:,2)<LonLatIn(2)));
    case 4
        TemmarkPt=find((T.LonLat(:,1)>=LonLatIn(1))&(T.LonLat(:,2)<LonLatIn(2)));
end
%
if numel(TemmarkPt)>0,
    TemLon=LonLatIn(1)*ones(numel(TemmarkPt),1);
    TemLat=LonLatIn(2)*ones(numel(TemmarkPt),1);
    [TemDist,a12,a21] = m_idist(TemLon,TemLat,T.LonLat(TemmarkPt,1),T.LonLat(TemmarkPt,2),'wgs84');
    [MinDist,MinPos]=min(abs(TemDist));
    PtIndex=TemmarkPt(MinPos);
    if (isnan(T.U(PtIndex,TimeStep))|isnan(T.V(PtIndex,TimeStep))),
        TemInd=1:numel(TemmarkPt);
        TemInd(MinPos)=NaN;
        TemInd=find(~isnan(TemInd));
        if numel(TemInd)>0,
            TemmarkPt=TemmarkPt(TemInd);
            TemLon=LonLatIn(1)*ones(numel(TemmarkPt),1);
            TemLat=LonLatIn(2)*ones(numel(TemmarkPt),1);
            [TemDist,a12,a21] = m_idist(TemLon,TemLat,T.LonLat(TemmarkPt,1),T.LonLat(TemmarkPt,2),'wgs84');
            [MinDist,MinPos]=min(abs(TemDist));
            PtIndex=TemmarkPt(MinPos);
            if (isnan(T.U(PtIndex,TimeStep))|isnan(T.V(PtIndex,TimeStep))),
                PtIndex=NaN;
            end
        else
            PtIndex=NaN;
        end
    end
else
    PtIndex=NaN;
end
%
%  the routine is done
%
return;
%
