function [ContourOut]=makeContour(ContourFile,ContourDepth,ContourSpacing,BathFile)
%
%  [ContourOut]=makeContour(ContourFile,ContourDepth,ContourSpacing,BathFile)
%
%  Inputs:  
%
%  ContourFile is the full path name, ContourDepth is the depth in meters that 
%  the user wishes to extract ,
%  and ContourSpacing is how often we would like points along the contour
%  (in kilometers).  Set to NaN to simply return the output using the default
%  spacing of the Matlab contour routine, whatever that happens to be.
%
%  BathFile is the file that has the coastal grids, used to make the CODAR
%  total vector plots.  If omitted, it is not shown when choosing the
%  correct contour.  If included, it is thrown up on the screen when
%  choosing.  The lat and lon limits for the plot are calculated automatically 
%  based upon the ContourFile and the Bathymetry file.  Lambert projection is used.
%
%  Outputs:
%
%  ContourOut is a 2 column matrix, column 1 is longitude, column 2 is
%  latitude
%
%  Countour file format is lon, lat, elevation (ascii)
%
%  the test files for the development:
%
%  ContourDirectory='/Users/chrishalle/Documents/CMHToolbox/HFR_Progs_Supplement/';
%  ContourFileName='CMHBML07_0.xyz';
%  ContourFile=[ContourDirectory ContourFileName];
%
%  BathFile='/Users/chrishalle/Documents/CMHToolbox/NationalParkStudy/BBayC
%  odarCoastHRes.mat';
%
%  ContourDepth should be entered as a negative number for depths, and a
%  positive number for elevations!  In other words, for the 50 meter depth
%  contour, enter -50  !!!!!!
%
%  Versions:
%  original code, Chris Halle, July 2007
%
%ContourDirectory='/Users/chrishalle/Documents/CMHToolbox/HFR_Progs_Supplement/';
%ContourFileName='CMHBML07_0.xyz';
%  ContourFile=[ContourDirectory ContourFileName];
%
disp('reading file, please be patient')
pause(1);
%
TemData=load(ContourFile);
%
disp('contour file read')
%
UniqueLon=unique(TemData(:,1))';
UniqueLat=unique(TemData(:,2))';
%
TotLon=max(size(UniqueLon));
%TotMeas=max(size(LonCol));
TotLat=max(size(UniqueLat));
%
ElevMatrix=reshape(TemData(:,3),TotLon,TotLat);
%
ElevMatrix=ElevMatrix';  %longitude increasing across the columns
ElevMatrix=flipud(ElevMatrix);
%
latout=UniqueLat;
lonout=UniqueLon;
elevout=ElevMatrix;
%
%  Find the contour.  Use the built in function to do so, it doesn't throw
%  plots up at the screen...... to check - accurate as using m_contour?
%
[C]=contourc(lonout,latout,elevout,[ContourDepth ContourDepth]);
%
%  The contour matrix can return a single contour, or a number of contours,
%  depending on the structure of the input elevation matrix.  Let us set this up to select
%  just one.  (In other words, the contours can be contiguous or not).
%
%  Find how many contours were returned.  Put each contour into a separate
%  cell entry.
%
if isempty(C),
    disp('No contours found')
    disp('routine ending')
    ContourOut=NaN;
    return;
end
%
[NumCRows,NumCCols]=size(C);  % NumCRows will be 2
%
%  Get the contours
%
EndPoint=0;
CurrentContourNum=0;
CurrContourStart=1;
%
while EndPoint<NumCCols,
    CurrentContourNum=CurrentContourNum+1;
    NumConPoints=C(2,CurrContourStart);
    EndPoint=EndPoint+NumConPoints+1;
    ContourLon=C(1,CurrContourStart+1:EndPoint);
    ContourLat=C(2,CurrContourStart+1:EndPoint);
    ContoursFound{1,CurrentContourNum}=[ContourLon' ContourLat'];
    CurrContourStart=CurrContourStart+NumConPoints+1;
end
%
if numel(ContoursFound)>1,
    %
    disp('more than one contour was found')
    disp('you will be prompted for which contour to select')
    disp('each contour will be displayed, you will be asked which one to use')
    disp('hit return to continue')
    pause
    %
    if exist('BathFile','var'),
        %
        eval(['load ' BathFile])
        LonLim=[nanmin(ncst(:,1)) nanmax(ncst(:,1))];
        LatLim=[nanmin(ncst(:,2)) nanmax(ncst(:,2))];
        clear Area k ncst
        %
        for jcon=1:numel(ContoursFound),
            %
            MinLon=min([min(LonLim) min(ContoursFound{jcon}(:,1))]);
            MaxLon=max([max(LonLim) max(ContoursFound{jcon}(:,1))]);
            LonLim=[MinLon MaxLon];
            %
            MinLat=min([min(LatLim) min(ContoursFound{jcon}(:,2))]);
            MaxLat=max([max(LatLim) max(ContoursFound{jcon}(:,2))]);
            LatLim=[MinLat MaxLat];
            %
        end
        %
        %  Make a map of the region
        %
        TemFig=figure;
        plotBasemap(LonLim,LatLim,BathFile,'lambert', 'patch', [0.5 0.5 0.5] );
        hold
        %
        %  Now loop through and make a plot of the contours, one at a time,
        %  asking if that particular contour is the right one to choose.
        %
        CorrectContour=0;
        ContourNum=0;
        %
        while ~CorrectContour,
            %
            if ContourNum<numel(ContoursFound),
                %
                ContourNum=ContourNum+1;
                %
                TemHandle = m_plot(ContoursFound{ContourNum}(:,1),ContoursFound{ContourNum}(:,2),'ro');
                %
                ContourResponse=questdlg('Is this the correct contour','Correct?','Yes','No','Yes');
                %
                if isequal(ContourResponse,'Yes'),
                    CorrectContour=1;
                    ContourChosen=ContourNum;
                end
                %
                delete(TemHandle);
                %
            else
                ContourResponse=questdlg('No Contours Chosen.  Loop through again?','Try Again?','Yes','No','Yes');
                switch ContourResponse
                    case 'Yes'
                        ContourNum=0;
                    case 'No'
                        CorrectContour=1;
                        ContourOut=NaN;
                        return;
                end
            end
        end
        %
        %  at this point, the bathymetry file has been plotted and used to
        %  select the appropriate contour.  Let us perform the process for
        %  the case where the file has not been specified.
        %
    else
        %
        %  let us just use the latlims and lonlims of the contours to plot
        %  the contour map.
        %
        LonAllContours=ContoursFound{1}(:,1);
        LatAllContours=ContoursFound{1}(:,2);
        %
        for jcon=2:numel(ContoursFound),
            LonAllContours=[LonAllContours;ContoursFound{jcon}(:,1)];
            LatAllContours=[LatAllContours;ContoursFound{jcon}(:,2)];
        end
        %
        LonLim=[min(LonAllContours) max(LonAllContours)];
        LatLim=[min(LatAllContours) max(LatAllContours)];
        %
        TemFig=figure;
        m_proj('lambert','lon',LonLim,'lat',LatLim);
        m_grid('linewidth',1,'linestyle','--','tickdir','out','fontsize',12);
        hold
        %
        %  now loop through and select the contour
        %
        CorrectContour=0;
        ContourNum=0;
        %
        while ~CorrectContour,
            %
            if ContourNum<numel(ContoursFound),
                %
                ContourNum=ContourNum+1;
                %
                TemHandle = m_plot(ContoursFound{ContourNum}(:,1),ContoursFound{ContourNum}(:,2),'ro');
                %
                ContourResponse=questdlg('Is this the correct contour','Correct?','Yes','No','Yes');
                %
                if isequal(ContourResponse,'Yes'),
                    CorrectContour=1;
                    ContourChosen=ContourNum;
                end
                %
                delete(TemHandle);
                %
            else
                ContourResponse=questdlg('No Contours Chosen.  Loop through again?','Try Again?','Yes','No','Yes');
                switch ContourResponse
                    case 'Yes'
                        ContourNum=0;
                    case 'No'
                        CorrectContour=1;
                        ContourOut=NaN;
                        return;
                end
            end
        end
    end
    %
    %  at this point, we have selected a single contour from all of the
    %  available contours that have been found.
    %
    delete(TemFig);
    ContourOut=ContoursFound{ContourChosen};
    %
else
    %
    %  only one contour found, so plot it.  Forget about map for now b/c of
    %  time, refine later
    %
    LonAllContours=ContoursFound{1}(:,1);
    LatAllContours=ContoursFound{1}(:,2);
    %
    LonLim=[min(LonAllContours) max(LonAllContours)];
    LatLim=[min(LatAllContours) max(LatAllContours)];
    %
    TemFig=figure;
    m_proj('lambert','lon',LonLim,'lat',LatLim);
    m_grid('linewidth',1,'linestyle','--','tickdir','out','fontsize',12);
    hold
    %
    TemHandle = m_plot(ContoursFound{1}(:,1),ContoursFound{1}(:,2),'ro');
    %
    ContourResponse=questdlg('One Contour Found','One Contour','OK','OK','OK');
    %
    delete(TemHandle);
    %
    delete(TemFig);
    %
    ContourOut=ContoursFound{1};
    %
end
%
%  We have one contour at this point.  Let us ask if we should trim the
%  contour.
%
ContourResponse=questdlg('Do you wish to trim the contour start and end?','Contour Trim','Yes','No','Yes');
if isequal(ContourResponse,'Yes'),
    if exist('BathFile','var'),
        TemFig=figure;
        plotBasemap(LonLim,LatLim,BathFile,'lambert', 'patch', [0.5 0.5 0.5] );
        hold
        m_plot(ContourOut(:,1),ContourOut(:,2),'ro');
        title('Contour in red')
    else
        TemFig=figure;
        m_proj('lambert','lon',LonLim,'lat',LatLim);
        m_grid('linewidth',1,'linestyle','--','tickdir','out','fontsize',12);
        hold
        m_plot(ContourOut(:,1),ContourOut(:,2),'ro');
        title('Contour in red')
    end
    %
    XAXLIMITS=get(gca,'XLim');
    YAXLIMITS=get(gca,'YLim');
    %
    InfoAboutZoom=questdlg('Do you wish to zoom in to select the first point?','Zoom In?','Yes','No','Yes');
    %
    switch InfoAboutZoom,
        case 'Yes'
            StopZoom=0;
        case 'No'
            StopZoom=1;
    end
    %
    while ~StopZoom,
        %
        InfoOnly=questdlg('Select lower left and upper right corner for zoom','Zoom In?','OK','OK','OK');
        %
        [xtem,ytem]=ginput(2);
        xtem=sort(xtem');
        ytem=sort(ytem');
        %
        set(gca,'XLim',xtem);
        set(gca,'YLim',ytem);
        %
        InfoAboutZoom=questdlg('Does the zoom look o.k.?','Zoom OK?','Yes','No','Yes');
        %
        if isequal(InfoAboutZoom,'Yes'),
            StopZoom=1;
        else
            set(gca,'XLim',XAXLIMITS);
            set(gca,'YLim',YAXLIMITS);
        end
    end
    %
    %  now pick the first point
    %
    InfoOnly=questdlg('Select first point','Select First Point','OK','OK','OK');
    %
    StopSelect=0;
    %
    while ~StopSelect,
        %
        [xtem,ytem]=m_ginput(1);
        %
        [DistFromPoint,a12,a21] = m_idist(xtem,ytem,ContourOut(:,1),ContourOut(:,2),'wgs84');
        temmark=find(isnan(DistFromPoint));  % if we happen to hit it exactly, correct for that
        if numel(temmark)>0,
            DistFromPoint(temmark)=0;
        end
        [MinVal,PosVal]=min(abs(DistFromPoint));
        %
        FirstPointLon=ContourOut(PosVal,1);
        FirstPointLat=ContourOut(PosVal,2);
        FirstPointPos=PosVal;
        %
        TemHandle=m_plot(FirstPointLon,FirstPointLat,'g+');
        %
        InfoAboutSelection=questdlg('Does the point selection look o.k.?','Point Selected?','Yes','No','Yes');
        %
        if isequal(InfoAboutSelection,'Yes'),
            StopSelect=1;
            set(gca,'XLim',XAXLIMITS);
            set(gca,'YLim',YAXLIMITS);
        else
            delete(TemHandle);
        end
    end
    %
    %  now repeat the whole thing for the last point
    %
    InfoAboutZoom=questdlg('Do you wish to zoom in to select the last point?','Zoom In?','Yes','No','Yes');
    %
    switch InfoAboutZoom,
        case 'Yes'
            StopZoom=0;
        case 'No'
            StopZoom=1;
    end
    %
    while ~StopZoom,
        %
        InfoOnly=questdlg('Select lower left and upper right corner for zoom','Zoom In?','OK','OK','OK');
        %
        [xtem,ytem]=ginput(2);
        xtem=sort(xtem');
        ytem=sort(ytem');
        %
        set(gca,'XLim',xtem);
        set(gca,'YLim',ytem);
        %
        InfoAboutZoom=questdlg('Does the zoom look o.k.?','Zoom OK?','Yes','No','Yes');
        %
        if isequal(InfoAboutZoom,'Yes'),
            StopZoom=1;
        else
            set(gca,'XLim',XAXLIMITS);
            set(gca,'YLim',YAXLIMITS);
        end
    end
    %
    %  now pick the last point
    %
    InfoOnly=questdlg('Select last point','Select Last Point','OK','OK','OK');
    %
    StopSelect=0;
    %
    while ~StopSelect,
        %
        [xtem,ytem]=m_ginput(1);
        %
        [DistFromPoint,a12,a21] = m_idist(xtem,ytem,ContourOut(:,1),ContourOut(:,2),'wgs84');
        temmark=find(isnan(DistFromPoint));  % if we happen to hit it exactly, correct for that
        if numel(temmark)>0,
            DistFromPoint(temmark)=0;
        end
        [MinVal,PosVal]=min(abs(DistFromPoint));
        %
        LastPointLon=ContourOut(PosVal,1);
        LastPointLat=ContourOut(PosVal,2);
        LastPointPos=PosVal;
        %
        TemHandle=m_plot(LastPointLon,LastPointLat,'g+');
        %
        InfoAboutSelection=questdlg('Does the point selection look o.k.?','Point Selected?','Yes','No','Yes');
        %
        if isequal(InfoAboutSelection,'Yes'),
            StopSelect=1;
            set(gca,'XLim',XAXLIMITS);
            set(gca,'YLim',YAXLIMITS);
        else
            delete(TemHandle);
        end
    end
    %
    %  we now have the positions of the first and the last point
    %
    ContourOut=ContourOut(FirstPointPos:LastPointPos,:);
    %
    delete(TemFig);
    %
end
%
%  we have one contour at this point.  Let us calculate the disance between
%  contour points, and then get the integrated distance along the contour.
%
[numCrows,numCcols]=size(ContourOut);
[DistAlongContour,a12,a21] = m_idist(ContourOut(1:numCrows-1,1),ContourOut(1:numCrows-1,2),ContourOut(2:numCrows,1),ContourOut(2:numCrows,2),'wgs84');
DistAlongContour=[0;DistAlongContour];
%
%  if some of the points in the contour are the same, then a NaN will
%  occur.  Search DistAlongContour for NaNs and remove from this vector and
%  the ContourOut matrix
%
temmark=find(~isnan(DistAlongContour));
DistAlongContour=DistAlongContour(temmark);
ContourOut=ContourOut(temmark,:);
DistAlongContour=(1/1000)*DistAlongContour;  % now in kilometers
DistAlongContour=cumsum(DistAlongContour);
%
%  one problem is that the DistanceOut Vector does not always have unique
%  entries.  Take care of that, and trim the contour matrix to match
%  (No - old way of doing things)
%
%[B,I,J]=unique(DistAlongContour);
%DistAlongContour=DistAlongContour(I);
%ContourOut=ContourOut(I,:);
%
if ~isnan(ContourSpacing),
    %
    %  now interpolate to get the contour points at the correct spacing
    %
    SpacingVector=0:ContourSpacing:max(DistAlongContour);
    %
    %  one problem is that the DistanceOut Vector does not always have unique
    %  entries.  Take care of that, and trim the contour matrix to match
    %
    LonOut=interp1(DistAlongContour',ContourOut(:,1)',SpacingVector);
    LatOut=interp1(DistAlongContour',ContourOut(:,2)',SpacingVector);
    %
    ContourOut=[LonOut' LatOut'];
    %
end
%
%  we now have the contour with the correct spacing.  Plot it up for the
%  user to see.
%
if exist('BathFile','var'),
    TemFig=figure;
    plotBasemap(LonLim,LatLim,BathFile,'lambert', 'patch', [0.5 0.5 0.5] );
    hold
    m_plot(ContourOut(:,1),ContourOut(:,2),'ro');
    title('contours in red')
    pause(2);
else
    TemFig=figure;
    m_proj('lambert','lon',LonLim,'lat',LatLim);
    m_grid('linewidth',1,'linestyle','--','tickdir','out','fontsize',12);
    hold
    m_plot(ContourOut(:,1),ContourOut(:,2),'ro');
    title('contours in red')
    pause(2);
end
%
ContourResponse=questdlg('Desired contour in red.','Contour Generated','OK','OK','OK');
%
delete(TemFig);
%
return;
%
