function [TransectOut]=makeTransect(CoastFile,TransectSpacing,SiteCoordsStart)
%
%  [TransectOut]=makeTransect(CoastFile,TransectSpacing,SiteCoordsStart);
%
%  This function will output lon and lat points along a transect, with the spacing 
%  between points equal to TransectSpacing (in kilometers).  Generally used to draw lines 
%  perpendicular (or at some angle) to the coast, with points at some spacing, to calculate
%  upcoast / downcoast flow.
%
%  BathFile is the full pathname of the coast file, ususally the same one as for plotting HF Radar data.
%
%  SiteCoords, if specified, are the start coordinates ([Lon Lat]) of the site to use near the coast.  If not specifed, 
%  the user is prompted graphically.
%
%  
%  There are a few ways to specify the transect in this program, which is prompt driven.
%   - The user generally selects the points graphically near the coast, and then out at sea.  These are
%     then adjusted.
%  - The user can choose to have thr transect "start" at the exact specified point near the coast, or at the closest 
%    point directly on the coast.
%  - The user can then choose to have the transect in the direction of the endpoint, or in a specified direction,
%  with the length of the transect equal to the distance between the specified start and end points.
%  - The length is adjusted so that the length divided by the spacing has no remainder (in other words,
%  we end up with a whole number of equally spaced points);
%
%  wgs 84 geoid and m_plot and m_idist used
%
%  Get limits of plot from bathymetry file
%
eval(['load ' CoastFile])
LonLim=[nanmin(ncst(:,1)) nanmax(ncst(:,1))];
LatLim=[nanmin(ncst(:,2)) nanmax(ncst(:,2))];
CoastalPoints=ncst;  % may be used later for distance calculations
clear Area k ncst
%
%  make the map
%
TemFig=figure;
plotBasemap(LonLim,LatLim,CoastFile,'lambert', 'patch', [0.5 0.5 0.5] );
hold
%
%  Set up the start and end points
%
if exist('SiteCoordsStart','var'),
    m_plot(SiteCoordsStart(1),SiteCoordsStart(2),'k*','MarkerSize',[14])
    title('chosen start site is a black star')
    %
    InfoOnly=questdlg('Select approx. location of transect end','Transect End','OK','OK','OK');
    %
    SiteOk=0;
    %
    while ~SiteOk,
        %
        [xtem,ytem]=m_ginput(1);
        TemEnd=m_plot(xtem,ytem,'ro','MarkerSize',[14]);
        EndSiteCheck=questdlg('Is the end site ok?','SiteEnd','Yes','No','Yes');
        if isequal(EndSiteCheck,'Yes'),
            SiteCoordsEnd=[xtem ytem];
            SiteOk=1;
        else
            delete(TemEnd);
        end
        %
    end
    %
else
    %
    InfoOnly=questdlg('Select approx. location of transect start and end','Transect End','OK','OK','OK');
    %
    InfoOnly=questdlg('Start is first point selected','Start Info','OK','OK','OK');
    %
    SiteOk=0;
    %
    while ~SiteOk,
        %
        [xtem,ytem]=m_ginput(2);
        TemStart=m_plot(xtem(1),ytem(1),'ro','MarkerSize',[14]);
        TemEnd=m_plot(xtem(2),ytem(2),'ro','MarkerSize',[14]);
        SiteCheck=questdlg('Are the transect end points o.k.?','EndPoints Accurate?','Yes','No','Yes');
        if isequal(SiteCheck,'Yes'),
            SiteCoordsStart=[xtem(1) ytem(1)];
            SiteCoordsEnd=[xtem(2) ytem(2)];
            SiteOk=1;
        else
            delete(TemStart);
            delete(TemEnd);
        end
        %
    end
    %
end
%
%  we now have the approximate start and end points
%
%  we now need to know if:  The user would like to use the actual start point indicated, use the closest coastal
%  point, or use the intersection point of the indicated transect with the coast
%
ModifyStartPointList=cell(4,1);
ModifyStartPointList{1}='Use Start Point';
ModifyStartPointList{2}='Use Closest Coastal Point';
%ModifyStartPointList{3}='Use Intersection of Transect with Coast';  % don't implement this one for now
ModifyStartPointList{3}='NOT IMPLEMENTED, DO NOT CHOOSE';
ModifyStartPointList{4}='Closest Coastal Point, but some distance offshore';
%
[Selection,OK]=listdlg('PromptString','Select A Start Point Method:','Name','StartPointMethod',...
    'SelectionMode','single','ListString',ModifyStartPointList);
if ~OK,
    disp('a start point method not chosen')
    disp('will default to no start point modification')
    StartPointSelection=1;
else
    StartPointSelection=Selection;
end
%
switch StartPointSelection
    case 1 % unmodified
        ModifiedStartPoint=SiteCoordsStart;
    case 2 % closest coastal point
        TemLon=SiteCoordsStart(1)*ones(size(CoastalPoints(:,1)));
        TemLat=SiteCoordsStart(2)*ones(size(CoastalPoints(:,1)));
        [TemDist,a12,a21] = m_idist(TemLon,TemLat,CoastalPoints(:,1),CoastalPoints(:,2),'wgs84');
        [MinDist,MinPos]=min(abs(TemDist));
        ModifiedStartPoint=[CoastalPoints(MinPos,1) CoastalPoints(MinPos,2)];
    case 3  % interesection
        %  find the intersection
    case 4 % closest point, but some distance offshore
        % just get the closest point for now....  get the distance offshore in another step
        %  But prompt for th edistance here!
        TemLon=SiteCoordsStart(1)*ones(size(CoastalPoints(:,1)));
        TemLat=SiteCoordsStart(2)*ones(size(CoastalPoints(:,1)));
        [TemDist,a12,a21] = m_idist(TemLon,TemLat,CoastalPoints(:,1),CoastalPoints(:,2),'wgs84');
        [MinDist,MinPos]=min(abs(TemDist));
        ModifiedStartPoint=[CoastalPoints(MinPos,1) CoastalPoints(MinPos,2)];
        %
        prompt={'Enter the distance offshore to start the transect:'};
        name='Offshore Distance (km):';
        numlines=1;
        defaultanswer={'0'};
        xlimitscell=inputdlg(prompt,name,numlines,defaultanswer);
        DistToStartTransect=[str2num(char(xlimitscell{1}))];
        clear prompt name numlines defaultanswer xlimitscell
        %
end
%
%  calculate the distance from the start point to the end point
%
[DistBetweenPoints,a12,a21] = m_idist(SiteCoordsEnd(1),SiteCoordsEnd(2),SiteCoordsStart(1),SiteCoordsStart(2),'wgs84');
DistBetweenPoints=DistBetweenPoints/1000;  % now in kilometers
AzStartToEnd=a21;
%
%  now prompt the user for how to handle the end point of the transect
%
ModifyEndPointList=cell(2,1);
ModifyEndPointList{1}='Use Bearing From Start to End Point';
ModifyEndPointList{2}='Prompt for a Bearing';
%
[Selection,OK]=listdlg('PromptString','Select a method to determine the transect bearing','Name','EndPointMethod',...
    'SelectionMode','single','ListString',ModifyEndPointList);
if (OK&(Selection==2)),
    BearingSelection=Selection;
    prompt={'Enter the bearing (CW from North) from start to end of transect'};
    name='Transect Bearing (deg):';
    numlines=1;
    defaultanswer={'0'};
    xlimitscell=inputdlg(prompt,name,numlines,defaultanswer);
    BearingToUse=[str2num(char(xlimitscell{1}))];
    clear prompt name numlines defaultanswer xlimitscell
else
    BearingSelection=1;
    BearingToUse=AzStartToEnd;
end
%
%  If the user chose to start at some distance from the nearest coastal point,
%  calculate the new start point now.
%
if isequal(StartPointSelection,4),
    %
    [lon2,lat2,a21] = m_fdist(ModifiedStartPoint(1),ModifiedStartPoint(2),BearingToUse,1000*DistToStartTransect,'wgs84');
    %
    %  check to see if we are n degrees west or east
    %
    if ModifiedStartPoint(1)<0,
        if lon2>0,
            lon2=lon2-360;
        end
    end
    %
    ModifiedStartPoint=[lon2 lat2];
    %
end
%
%   calculate the new distance between points, leave BearingToUse the same as we already have
%
[DistBetweenPoints,a12,a21] = m_idist(SiteCoordsEnd(1),SiteCoordsEnd(2),ModifiedStartPoint(1),ModifiedStartPoint(2),'wgs84');
DistBetweenPoints=DistBetweenPoints/1000;  % now in kilometers
%
%  now calculate a spacing matrix.....  add a little to the distance vector so that we are sure to include the endpoint of
%  the transect.  This isn;t extrenely rigorous, but doesn't need to be
%
SpacingVector=0:TransectSpacing:(DistBetweenPoints+TransectSpacing);  % in kilometers
SpacingVector=1000*SpacingVector;  % in meters
SpacingVector=SpacingVector';
%
NumPts=numel(SpacingVector);
%
%  now make sure we have degrees west if supplied in degrees west
%
[LonOut,LatOut,Az21out] = m_fdist(ModifiedStartPoint(1)*ones(NumPts,1),ModifiedStartPoint(2)*ones(NumPts,1),...
    BearingToUse*ones(NumPts,1),SpacingVector,'wgs84');
%
if ModifiedStartPoint(1)<0,
    if LonOut(1)>0,
        LonOut=LonOut-360;
    end
end
%
%  now plot them up
%
[TemHandleTransect]=m_plot(LonOut,LatOut,'r.','MarkerSize',[12])
%
InfoOnly=questdlg('Transect Points in red dots','Transect Points','OK','OK','OK');
InfoOnly=questdlg('hit return when done viewing figure','View Figure','OK','OK','OK');
%
pause
%
delete(TemFig);
%
TransectOut=[LonOut LatOut];
%
%  routine is done
%
return;
%
